# -*- coding: utf-8 -*-
"""
plotpy
======

Based on `PythonQwt` (plotting widgets for Qt graphical user interfaces) and
on the scientific modules NumPy and SciPy, :mod:`plotpy` is a Python library
providing efficient 2D data-plotting features (curve/image visualization
and related tools) for interactive computing and signal/image processing
application development.

.. image:: images/panorama.png


External resources:
    * Python Package Index: `PyPI`_
    * Bug reports and feature requests: `GitHub`_

.. _PyPI: https://pypi.python.org/pypi/plotpy
.. _GitHub: https://github.com/PierreRaybaut/plotpy
"""

__version__ = "2.7.0"
__VERSION__ = tuple([int(number) for number in __version__.split(".")])

# --- Important note: DATAPATH and LOCALEPATH are used by guidata.configtools
# ---                 to retrieve data and translation files paths
#
# Dear (Debian, RPM, ...) package makers, please feel free to customize the
# following path to module's data (e.g. icons) and translations:
DATAPATH = LOCALEPATH = ""



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

"""ActionTool test"""

# guitest: show

from guidata.qthelpers import create_action, qt_app_context
from qtpy import QtWidgets as QW

from plotpy.builder import make
from plotpy.items import ImageItem
from plotpy.plot import PlotDialog, PlotOptions
from plotpy.tests.data import gen_image4
from plotpy.tools import ActionTool


class MyPlotDialog(PlotDialog):
    def __init__(self):
        """Reimplement PlotDialog method"""
        super().__init__(
            title="ActionTool test", toolbar=True, options=PlotOptions(type="image")
        )
        self.info_action = create_action(self, "Show infos", triggered=self.show_info)
        self.manager.add_tool(ActionTool, self.info_action, item_types=(ImageItem,))

    def show_info(self):
        """Show infos on selected item(s)"""
        # This is just a demo of what can be done with ActionTool
        plot = self.get_plot()
        for item in plot.get_selected_items():
            QW.QMessageBox.information(self, "Item infos", str(item))


def test_image_plot_tools():
    """Test"""
    with qt_app_context(exec_loop=True):
        win = MyPlotDialog()
        win.show()
        image = make.image(gen_image4(500, 500), colormap="Spectral")
        plot = win.manager.get_plot()
        plot.add_item(image)


if __name__ == "__main__":
    test_image_plot_tools()



# -*- coding: utf-8 -*-

import os.path as osp
import sys

import numpy as np
from guidata.configtools import get_icon
from guidata.dataset import BeginGroup, BoolItem, ChoiceItem, DataSet, EndGroup
from guidata.qthelpers import get_std_icon
from qtpy import QtCore as QC
from qtpy import QtWidgets as QW
from qtpy.QtPrintSupport import QPrintDialog, QPrinter

from plotpy import io
from plotpy.config import _
from plotpy.interfaces import IImageItemType
from plotpy.items import (
    compute_trimageitems_original_size,
    get_image_from_plot,
    get_items_in_rectangle,
    get_plot_qrect,
)
from plotpy.tools.base import CommandTool, DefaultToolbarID, RectangularActionTool
from plotpy.widgets import about
from plotpy.widgets.resizedialog import ResizeDialog


class SaveAsTool(CommandTool):
    """ """

    def __init__(self, manager, toolbar_id=DefaultToolbarID):
        super().__init__(
            manager,
            _("Save as..."),
            get_std_icon("DialogSaveButton", 16),
            toolbar_id=toolbar_id,
        )

    def activate_command(self, plot, checked):
        """Activate tool"""
        # FIXME: Qt's PDF printer is unable to print plots including images
        # --> until this bug is fixed internally, disabling PDF output format
        #     when plot has image items.
        formats = "%s (*.png)" % _("PNG image")

        for item in plot.get_items():
            if IImageItemType in item.types():
                break
        else:
            formats += "\n%s (*.pdf)" % _("PDF document")
        fname, _f = QW.QFileDialog.getSaveFileName(
            plot, _("Save as"), _("untitled"), formats
        )
        if fname:
            plot.save_widget(fname)


class CopyToClipboardTool(CommandTool):
    """ """

    def __init__(self, manager, toolbar_id=DefaultToolbarID):
        super().__init__(
            manager,
            _("Copy to clipboard"),
            get_icon("copytoclipboard.png"),
            toolbar_id=toolbar_id,
        )

    def activate_command(self, plot, checked):
        """Activate tool"""
        plot.copy_to_clipboard()


def save_snapshot(plot, p0, p1, new_size=None):
    """
    Save rectangular plot area
    p0, p1: resp. top left and bottom right points (`QPointF` objects)
    new_size: destination image size (tuple: (width, height))
    """
    items = get_items_in_rectangle(plot, p0, p1)
    if not items:
        QW.QMessageBox.critical(
            plot,
            _("Rectangle snapshot"),
            _("There is no supported image item in current selection."),
        )
        return
    src_x, src_y, src_w, src_h = get_plot_qrect(plot, p0, p1).getRect()
    original_size = compute_trimageitems_original_size(items, src_w, src_h)

    if new_size is None:
        new_size = (int(p1.x() - p0.x() + 1), int(p1.y() - p0.y() + 1))  # Screen size

    dlg = ResizeDialog(
        plot, new_size=new_size, old_size=original_size, text=_("Destination size:")
    )
    if not dlg.exec():
        return

    class SnapshotParam(DataSet):
        _levels = BeginGroup(_("Image levels adjustments"))
        apply_contrast = BoolItem(_("Apply contrast settings"), default=False)
        apply_interpolation = BoolItem(_("Apply interpolation algorithm"), default=True)
        norm_range = BoolItem(_("Scale levels to maximum range"), default=False)
        _end_levels = EndGroup(_("Image levels adjustments"))
        _multiple = BeginGroup(_("Superimposed images"))
        add_images = ChoiceItem(
            _("If image B is behind image A, " "replace intersection by"),
            [(False, "A"), (True, "A+B")],
            default=None,
        )
        _end_multiple = EndGroup(_("Superimposed images"))

    param = SnapshotParam(_("Rectangle snapshot"))
    if not param.edit(parent=plot):
        return

    if dlg.keep_original_size:
        destw, desth = original_size
    else:
        destw, desth = dlg.width, dlg.height

    try:
        data = get_image_from_plot(
            plot,
            p0,
            p1,
            destw=destw,
            desth=desth,
            add_images=param.add_images,
            apply_lut=param.apply_contrast,
            apply_interpolation=param.apply_interpolation,
            original_resolution=dlg.keep_original_size,
        )

        dtype = None
        for item in items:
            if dtype is None or item.data.dtype.itemsize > dtype.itemsize:
                dtype = item.data.dtype
        if param.norm_range:
            data = io.scale_data_to_dtype(data, dtype=dtype)
        else:
            data = np.array(data, dtype=dtype)
    except MemoryError:
        mbytes = int(destw * desth * 32.0 / (8 * 1024**2))
        text = _(
            "There is not enough memory left to process "
            "this {destw:d} x {desth:d} image ({mbytes:d} "
            "MB would be required)."
        )
        text = text.format(destw=destw, desth=desth, mbytes=mbytes)
        QW.QMessageBox.critical(plot, _("Memory error"), text)
        return
    for model_item in items:
        model_fname = model_item.get_filename()
        if model_fname is not None and model_fname.lower().endswith(".dcm"):
            break
    else:
        model_fname = None
    fname, _f = QW.QFileDialog.getSaveFileName(
        plot,
        _("Save as"),
        _("untitled"),
        io.iohandler.get_filters("save", data.dtype, template=True),
    )
    _base, ext = osp.splitext(fname)
    options = {}
    if not fname:
        return

    elif ext.lower() == ".png":
        options.update(dict(dtype=np.uint8, max_range=True))

    elif ext.lower() == ".dcm":
        # This import statement must stay here because if pydicom is not installed,
        # the extension .dcm is not registered in the io module, so we will not
        # get here.
        try:
            from pydicom import dcmread  # pylint: disable=import-outside-toplevel
        except ImportError:
            raise ImportError("This should not happen (pydicom is not installed)")

        model_dcm = dcmread(model_fname)
        try:
            ps_attr = "ImagerPixelSpacing"
            ps_x, ps_y = getattr(model_dcm, ps_attr)
        except AttributeError:
            ps_attr = "PixelSpacing"
            ps_x, ps_y = getattr(model_dcm, ps_attr)
        model_dcm.Rows, model_dcm.Columns = data.shape

        dest_height, dest_width = data.shape
        (
            _x,
            _y,
            _angle,
            model_dx,
            model_dy,
            _hflip,
            _vflip,
        ) = model_item.get_transform()
        new_ps_x = ps_x * src_w / (model_dx * dest_width)
        new_ps_y = ps_y * src_h / (model_dy * dest_height)
        setattr(model_dcm, ps_attr, [new_ps_x, new_ps_y])
        options.update(dict(template=model_dcm))
    io.imwrite(fname, data, **options)


class SnapshotTool(RectangularActionTool):
    """ """

    SWITCH_TO_DEFAULT_TOOL = True
    TITLE = _("Rectangle snapshot")
    ICON = "snapshot.png"

    def __init__(self, manager, toolbar_id=DefaultToolbarID):
        super().__init__(
            manager, save_snapshot, toolbar_id=toolbar_id, fix_orientation=True
        )


class HelpTool(CommandTool):
    """ """

    def __init__(self, manager, toolbar_id=DefaultToolbarID):
        super().__init__(
            manager,
            _("Help"),
            get_std_icon("DialogHelpButton", 16),
            toolbar_id=toolbar_id,
        )

    def activate_command(self, plot, checked):
        """Activate tool"""
        info = _(
            """<b>Keyboard/mouse shortcuts:</b><br><br>
  - <u>single left-click</u>: item (curve, image, ...) selection<br>
  - <u>single right-click</u>: context-menu relative to selected item<br>
  - <u>shift</u>: on-active-curve (or image) cursor (+ <u>control</u> to maintain
cursor visible)<br>
  - <u>shift + control</u>: on-active-curve cursor (+ <u>control</u> to maintain
cursor visible)<br>
  - <u>alt</u>: free cursor<br>
  - <u>left-click + mouse move</u>: move item (when available)<br>
  - <u>middle-click + mouse move</u>: pan<br>
  - <u>right-click + mouse move</u>: zoom"""
        )
        info += "<br><hr><br><b>" + _("Information on PlotPy:") + "</b><br><br>"
        info += about.about(html=True)
        QW.QMessageBox.information(plot, _("Help"), info)


class AboutTool(CommandTool):
    """ """

    def __init__(self, manager, toolbar_id=DefaultToolbarID):
        super().__init__(
            manager, _("About") + " plotpy", get_icon("plotpy.svg"), toolbar_id=None
        )

    def activate_command(self, plot, checked):
        """Activate tool"""
        about.show_about_dialog()


class PrintTool(CommandTool):
    """ """

    def __init__(self, manager, toolbar_id=DefaultToolbarID):
        super().__init__(
            manager, _("Print..."), get_icon("print.png"), toolbar_id=toolbar_id
        )

    def activate_command(self, plot, checked):
        """Activate tool"""
        printer = QPrinter()
        dialog = QPrintDialog(printer, plot)
        saved_in, saved_out, saved_err = sys.stdin, sys.stdout, sys.stderr
        sys.stdout = None
        ok = dialog.exec()
        sys.stdin, sys.stdout, sys.stderr = saved_in, saved_out, saved_err
        if ok:
            plot.print_(printer)


class OpenFileTool(CommandTool):
    """ """

    #: Signal emitted by OpenFileTool when a file was opened
    #:
    #: Args:
    #:     filename (str): The name of the file that was opened
    SIG_OPEN_FILE = QC.Signal(str)

    def __init__(
        self, manager, title=_("Open..."), formats="*.*", toolbar_id=DefaultToolbarID
    ):
        super().__init__(
            manager, title, get_std_icon("DialogOpenButton", 16), toolbar_id=toolbar_id
        )
        self.formats = formats
        self.directory = ""

    def get_filename(self, plot):
        """

        :param plot:
        :return:
        """
        saved_in, saved_out, saved_err = sys.stdin, sys.stdout, sys.stderr
        sys.stdout = None
        filename, _f = QW.QFileDialog.getOpenFileName(
            plot, _("Open"), self.directory, self.formats
        )
        sys.stdin, sys.stdout, sys.stderr = saved_in, saved_out, saved_err
        filename = str(filename)
        if filename:
            self.directory = osp.dirname(filename)
        return filename

    def activate_command(self, plot, checked):
        """Activate tool"""
        filename = self.get_filename(plot)
        if filename:
            self.SIG_OPEN_FILE.emit(filename)


class FilterTool(CommandTool):
    """ """

    def __init__(self, manager, filter, toolbar_id=None):
        super().__init__(manager, str(filter.name), toolbar_id=toolbar_id)
        self.filter = filter

    def update_status(self, plot):
        """

        :param plot:
        """
        self.set_status_active_item(plot)

    def activate_command(self, plot, checked):
        """Activate tool"""
        plot.apply_filter(self.filter)



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

"""Annotation test"""

# guitest: show

from guidata.qthelpers import qt_app_context
from numpy import linspace, sin

from plotpy.builder import make
from plotpy.tests import data as ptd
from plotpy.tests import vistools as ptv


def plot(*items, plot_type="auto"):
    title = "All annotation tools"
    if plot_type in ("curve", "image"):
        title = f"{plot_type.capitalize()} specialized plot annotation tools"
    win = ptv.show_items(items, plot_type=plot_type, wintitle=title, title=title)
    win.register_annotation_tools()
    return win


def test_annotation():
    """Test annotation"""
    x = linspace(-10, 10, 200)
    y = sin(sin(sin(x)))
    persist = []
    with qt_app_context(exec_loop=True):
        persist.append(plot(make.curve(x, y, color="b"), plot_type="curve"))
        persist.append(plot(make.image(ptd.gen_image1()), plot_type="image"))
        persist.append(plot(make.curve(x, y, color="b"), make.image(ptd.gen_image1())))


if __name__ == "__main__":
    test_annotation()



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

# pylint: disable=C0103

"""
Curve/cursor/marker Item builder
--------------------------------

This module provides a set of factory functions to simplify the creation of
curve, cursor and marker items.
"""

# Note: when adding method to builder classes, please do not forget to update the
# documentation (see builder.rst file). Because of class inheritance, the methods
# are not automatically documented (otherwise, they would be sorted alphabetically,
# due to a limitation of sphinx auto-doc).

from __future__ import annotations

from typing import TYPE_CHECKING

import numpy  # only to help intersphinx finding numpy doc
import numpy as np

from plotpy.config import CONF, _, make_title
from plotpy.items import (
    CurveItem,
    ErrorBarCurveItem,
    HistogramItem,
    Marker,
    XRangeSelection,
)
from plotpy.plot import BasePlot
from plotpy.styles import (
    COLORS,
    MARKERS,
    CurveParam,
    ErrorBarParam,
    HistogramParam,
    MarkerParam,
    style_generator,
    update_style_attr,
)

if TYPE_CHECKING:
    from typing import Callable

CURVE_COUNT = 0
HISTOGRAM_COUNT = 0


class CurveMarkerCursorBuilder:
    """Class regrouping a set of factory functions to simplify the creation
    of curve, cursor and marker items."""

    def __init__(self):
        self.style = style_generator()

    # ---- Plot items -----------------------------------------------------------------

    def __set_curve_axes(self, curve: CurveItem, xaxis: str, yaxis: str) -> None:
        """Set curve axes"""
        for axis in (xaxis, yaxis):
            if axis not in BasePlot.AXIS_NAMES:
                raise RuntimeError(f"Unknown axis {axis}")
        curve.setXAxis(BasePlot.AXIS_NAMES[xaxis])
        curve.setYAxis(BasePlot.AXIS_NAMES[yaxis])

    def __set_baseparam(
        self,
        param: CurveParam | MarkerParam,
        color: str | None = None,
        linestyle: str | None = None,
        linewidth: int | None = None,
        marker: str | None = None,
        markersize: int | None = None,
        markerfacecolor: str | None = None,
        markeredgecolor: str | None = None,
    ) -> None:
        """Apply parameters to a :py:class:`.CurveParam` or
        :py:class:`.MarkerParam` instance"""
        if color is not None:
            color = COLORS.get(color, color)  # MATLAB-style
            param.line.color = color
        if linestyle is not None:
            param.line.set_style_from_matlab(linestyle)
        if linewidth is not None:
            param.line.width = linewidth
        if marker is not None:
            if marker in MARKERS:
                param.symbol.update_param(MARKERS[marker])  # MATLAB-style
            else:
                param.symbol.marker = marker
        if markersize is not None:
            param.symbol.size = markersize
        if markerfacecolor is not None:
            markerfacecolor = COLORS.get(
                markerfacecolor, markerfacecolor
            )  # MATLAB-style
            param.symbol.facecolor = markerfacecolor
        if markeredgecolor is not None:
            markeredgecolor = COLORS.get(
                markeredgecolor, markeredgecolor
            )  # MATLAB-style
            param.symbol.edgecolor = markeredgecolor

    def __set_param(
        self,
        param: CurveParam,
        title: str | None = None,
        color: str | None = None,
        linestyle: str | None = None,
        linewidth: int | None = None,
        marker: str | None = None,
        markersize: int | None = None,
        markerfacecolor: str | None = None,
        markeredgecolor: str | None = None,
        shade: bool | None = None,
        curvestyle: str | None = None,
        baseline: float | None = None,
        dsamp_factor: int | None = None,
        use_dsamp: bool | None = None,
    ) -> None:
        """Apply parameters to a :py:class:`.CurveParam` instance"""
        self.__set_baseparam(
            param,
            color,
            linestyle,
            linewidth,
            marker,
            markersize,
            markerfacecolor,
            markeredgecolor,
        )
        if title:
            param.label = title
        if shade is not None:
            param.shade = shade
        if curvestyle is not None:
            param.curvestyle = curvestyle
        if baseline is not None:
            param.baseline = baseline
        if dsamp_factor is not None:
            param.dsamp_factor = dsamp_factor
        if use_dsamp is not None:
            param.use_dsamp = use_dsamp

    def __get_arg_triple_plot(self, args):
        """Convert MATLAB-like arguments into x, y, style"""

        def get_x_y_from_data(data):
            """

            :param data:
            :return:
            """
            if isinstance(data, (tuple, list)):
                data = np.array(data)
            if len(data.shape) == 1 or 1 in data.shape:
                x = np.arange(data.size)
                y = data
            else:
                x = np.arange(len(data[:, 0]))
                y = [data[:, i] for i in range(len(data[0, :]))]
            return x, y

        if len(args) == 1:
            if isinstance(args[0], str):
                x = np.array((), float)
                y = np.array((), float)
                style = args[0]
            else:
                x, y = get_x_y_from_data(args[0])
                if not isinstance(y, np.ndarray):
                    style = [next(self.style) for yi in y]
                else:
                    style = next(self.style)
        elif len(args) == 2:
            a1, a2 = args
            if isinstance(a2, str):
                x, y = get_x_y_from_data(a1)
                style = a2
            else:
                x = a1
                y = a2
                style = next(self.style)
        elif len(args) == 3:
            x, y, style = args
        else:
            raise TypeError("Wrong number of arguments")
        if isinstance(x, (list, tuple)):
            x = np.array(x)
        if isinstance(y, (list, tuple)) and isinstance(y, np.ndarray):
            y = np.array(y)
        return x, y, style

    def __get_arg_triple_errorbar(self, args):
        """Convert MATLAB-like arguments into x, y, style"""
        if len(args) == 2:
            y, dy = args
            x = np.arange(len(y))
            dx = np.zeros(len(y))
            style = next(self.style)
        elif len(args) == 3:
            a1, a2, a3 = args
            if isinstance(a3, str):
                y, dy = a1, a2
                x = np.arange(len(y))
                dx = np.zeros(len(y))
                style = a3
            else:
                x, y, dy = args
                dx = np.zeros(len(y))
                style = next(self.style)
        elif len(args) == 4:
            a1, a2, a3, a4 = args
            if isinstance(a4, str):
                x, y, dy = a1, a2, a3
                dx = np.zeros(len(y))
                style = a4
            else:
                x, y, dx, dy = args
                style = next(self.style)
        elif len(args) == 5:
            x, y, dx, dy, style = args
        else:
            raise TypeError("Wrong number of arguments")
        return x, y, dx, dy, style

    def mcurve(self, *args, **kwargs) -> CurveItem | list[CurveItem]:
        """Make a curve `plot item` based on MATLAB-like syntax
        (may returns a list of curves if data contains more than one signal)

        Args:
            args: x, y, style
            kwargs: title, color, linestyle, linewidth, marker, markersize,
            markerfacecolor, markeredgecolor, shade, curvestyle, baseline,
            dsamp_factor, use_dsamp

        Returns:
            :py:class:`.CurveItem` object

        Example::
            mcurve(x, y, 'r+')
        """  # noqa: E501
        x, y, style = self.__get_arg_triple_plot(args)
        if isinstance(y, np.ndarray):
            y = [y]
        if not isinstance(style, list):
            style = [style]
        if len(y) > len(style):
            style = [style[0]] * len(y)
        basename = _("Curve")
        curves = []
        for yi, stylei in zip(y, style):
            param = CurveParam(title=basename, icon="curve.png")
            if "label" in kwargs:
                param.label = kwargs.pop("label")
            else:
                global CURVE_COUNT
                CURVE_COUNT += 1
                param.label = make_title(basename, CURVE_COUNT)
            if "dsamp_factor" in kwargs:
                param.dsamp_factor = kwargs.pop("dsamp_factor")
            if "use_dsamp" in kwargs:
                param.use_dsamp = kwargs.pop("use_dsamp")
            update_style_attr(stylei, param)
            curves.append(self.pcurve(x, yi, param, **kwargs))
        if len(curves) == 1:
            return curves[0]
        else:
            return curves

    def pcurve(
        self,
        x: numpy.ndarray,
        y: numpy.ndarray,
        param: CurveParam,
        xaxis: str = "bottom",
        yaxis: str = "left",
    ) -> CurveItem:
        """Make a curve `plot item` based on a :py:class:`.CurveParam` instance

        Args:
            x: x data
            y: y data
            param: curve parameters
            xaxis: x axis name. Default is 'bottom'
            yaxis: y axis name. Default is 'left'

        Returns:
            :py:class:`.CurveItem` object

        Example::
            pcurve(x, y, param)
        """
        curve = CurveItem(param)
        curve.set_data(x, y)
        curve.update_params()
        self.__set_curve_axes(curve, xaxis, yaxis)
        return curve

    def curve(
        self,
        x: numpy.ndarray,
        y: numpy.ndarray,
        title: str = "",
        color: str | None = None,
        linestyle: str | None = None,
        linewidth: int | None = None,
        marker: str | None = None,
        markersize: int | None = None,
        markerfacecolor: str | None = None,
        markeredgecolor: str | None = None,
        shade: bool | None = None,
        curvestyle: str | None = None,
        baseline: float | None = None,
        xaxis: str = "bottom",
        yaxis: str = "left",
        dsamp_factor: int | None = None,
        use_dsamp: bool | None = None,
        dx: numpy.ndarray | None = None,
        dy: numpy.ndarray | None = None,
        errorbarwidth: int | None = None,
        errorbarcap: int | None = None,
        errorbarmode: str | None = None,
        errorbaralpha: float | None = None,
    ) -> CurveItem:
        """Make a curve `plot item` from x, y, data

        Args:
            x: x data
            y: y data
            title: curve title. Default is ''
            color: curve color name. Default is None
            linestyle: curve line style (MATLAB-like string or "SolidLine",
             "DashLine", "DotLine", "DashDotLine", "DashDotDotLine", "NoPen").
             Default is None
            linewidth: line width (pixels). Default is None
            marker: marker shape (MATLAB-like string or "Cross",
             "Ellipse", "Star1", "XCross", "Rect", "Diamond", "UTriangle",
             "DTriangle", "RTriangle", "LTriangle", "Star2", "NoSymbol").
             Default is None
            markersize: marker size (pixels). Default is None
            markerfacecolor: marker face color name. Default is None
            markeredgecolor: marker edge color name. Default is None
            shade: 0 <= float <= 1 (curve shade). Default is None
            curvestyle: "Lines", "Sticks", "Steps", "Dots", "NoCurve".
             Default is None
            baseline: baseline value. Default is None
            xaxis: x axis name. Default is 'bottom'
            yaxis: y axis name. Default is 'left'
            dsamp_factor: downsampling factor. Default is None
            use_dsamp: use downsampling. Default is None
            dx: x error data. Default is None
            dy: y error data. Default is None
            errorbarwidth: error bar width (pixels). Default is None
            errorbarcap: error bar cap size (pixels). Default is None
            errorbarmode: error bar mode ("Vertical", "Horizontal",
             "Both"). Default is None
            errorbaralpha: 0 <= float <= 1 (error bar transparency).
             Default is None

        Returns:
            :py:class:`.CurveItem` object

        Example::
            curve(x, y, marker='Ellipse', markerfacecolor='#ffffff')

        which is equivalent to (MATLAB-style support)::

            curve(x, y, marker='o', markerfacecolor='w')
        """

        if dx is not None or dy is not None:
            return self.error(
                x,
                y,
                dx,
                dy,
                title=title,
                color=color,
                linestyle=linestyle,
                linewidth=linewidth,
                errorbarwidth=errorbarwidth,
                errorbarcap=errorbarcap,
                errorbarmode=errorbarmode,
                errorbaralpha=errorbaralpha,
                marker=marker,
                markersize=markersize,
                markerfacecolor=markerfacecolor,
                markeredgecolor=markeredgecolor,
                shade=shade,
                curvestyle=curvestyle,
                baseline=baseline,
                xaxis=xaxis,
                yaxis=yaxis,
                dsamp_factor=dsamp_factor,
                use_dsamp=use_dsamp,
            )

        basename = _("Curve")
        param = CurveParam(title=basename, icon="curve.png")
        if not title:
            global CURVE_COUNT
            CURVE_COUNT += 1
            title = make_title(basename, CURVE_COUNT)
        self.__set_param(
            param,
            title,
            color,
            linestyle,
            linewidth,
            marker,
            markersize,
            markerfacecolor,
            markeredgecolor,
            shade,
            curvestyle,
            baseline,
            dsamp_factor,
            use_dsamp,
        )
        return self.pcurve(x, y, param, xaxis, yaxis)

    def merror(self, *args, **kwargs) -> ErrorBarCurveItem:
        """Make an errorbar curve `plot item` based on MATLAB-like syntax

        Args:
            args: x, y, dx, dy, style
            kwargs: title, color, linestyle, linewidth, marker, markersize,
             markerfacecolor, markeredgecolor, shade, curvestyle, baseline,
             xaxis, yaxis, errorbarwidth, errorbarcap, errorbarmode,
             errorbaralpha

        Returns:
            :py:class:`.ErrorBarCurveItem` object

        Example::
            mcurve(x, y, 'r+')
        """  # noqa: E501
        x, y, dx, dy, style = self.__get_arg_triple_errorbar(args)
        basename = _("Curve")
        curveparam = CurveParam(title=basename, icon="curve.png")
        errorbarparam = ErrorBarParam(title=_("Error bars"), icon="errorbar.png")
        if "label" in kwargs:
            curveparam.label = kwargs["label"]
        else:
            global CURVE_COUNT
            CURVE_COUNT += 1
            curveparam.label = make_title(basename, CURVE_COUNT)
        update_style_attr(style, curveparam)
        errorbarparam.color = curveparam.line.color
        return self.perror(x, y, dx, dy, curveparam, errorbarparam)

    def perror(
        self,
        x: numpy.ndarray,
        y: numpy.ndarray,
        dx: numpy.ndarray,
        dy: numpy.ndarray,
        curveparam: CurveParam,
        errorbarparam: ErrorBarParam,
        xaxis: str = "bottom",
        yaxis: str = "left",
    ) -> ErrorBarCurveItem:
        """Make an errorbar curve `plot item`
        based on a :py:class:`.ErrorBarParam` instance

        Args:
            x: x data
            y: y data
            dx: x error data
            dy: y error data
            curveparam: curve style
            errorbarparam: error bar style
            xaxis: x axis name. Default is 'bottom'
            yaxis: y axis name. Default is 'left'

        Returns:
            :py:class:`.ErrorBarCurveItem` object

        Example::
            perror(x, y, dx, dy, curveparam, errorbarparam)
        """
        curve = ErrorBarCurveItem(curveparam, errorbarparam)
        curve.set_data(x, y, dx, dy)
        curve.update_params()
        self.__set_curve_axes(curve, xaxis, yaxis)
        return curve

    def error(
        self,
        x: numpy.ndarray,
        y: numpy.ndarray,
        dx: numpy.ndarray,
        dy: numpy.ndarray,
        title: str = "",
        color: str | None = None,
        linestyle: str | None = None,
        linewidth: float | None = None,
        errorbarwidth: int | None = None,
        errorbarcap: int | None = None,
        errorbarmode: str | None = None,
        errorbaralpha: float | None = None,
        marker: str | None = None,
        markersize: float | None = None,
        markerfacecolor: str | None = None,
        markeredgecolor: str | None = None,
        shade: bool | None = None,
        curvestyle: str | None = None,
        baseline: float | None = None,
        xaxis: str = "bottom",
        yaxis: str = "left",
        dsamp_factor: int | None = None,
        use_dsamp: bool | None = None,
    ) -> ErrorBarCurveItem:
        """Make an errorbar curve `plot item`

        Args:
            x: x data
            y: y data
            dx: x error data
            dy: y error data
            title: curve title. Default is ''
            color: curve color name. Default is None
            linestyle: curve line style (MATLAB-like string or attribute
             name from the `PyQt5.QtCore.Qt.PenStyle` enum
             (i.e. "SolidLine" "DashLine", "DotLine", "DashDotLine",
             "DashDotDotLine" or "NoPen"). Default is None
            linewidth: line width (pixels). Default is None
            errorbarwidth: error bar width (pixels). Default is None
            errorbarcap: error bar cap length (pixels). Default is None
            errorbarmode: error bar mode (MATLAB-like string or attribute
             name from the `qwt.QwtPlotCurve.ErrorBar` enum
             (i.e. "NoError", "ErrorBar", "ErrorSymbol", "ErrorBarSymbol",
             "ErrorBarCurve"). Default is None
            errorbaralpha: error bar alpha value (0.0 transparent
             through 1.0 opaque). Default is None
            marker: marker shape (MATLAB-like string or attribute name
             from the `qwt.QwtSymbol.Style` enum (i.e. "Cross",
             "Ellipse", "Star1", "XCross", "Rect", "Diamond", "UTriangle",
             "DTriangle", "RTriangle", "LTriangle", "NoSymbol"). Default is None
            markersize: marker size (pixels). Default is None
            markerfacecolor: marker face color name. Default is None
            markeredgecolor: marker edge color name. Default is None
            shade: shade under curve. Default is None
            curvestyle: curve style (MATLAB-like string or attribute name
             from the `qwt.QwtPlotCurve.CurveStyle` enum (i.e. "NoCurve",
             "Lines", "Sticks", "Steps", "Dots"). Default is None
            baseline: baseline value. Default is None
            xaxis: x axis name. Default is 'bottom'
            yaxis: y axis name. Default is 'left'
            dsamp_factor: downsampling factor. Default is None
            use_dsamp: use downsampling. Default is None

        Returns:
            :py:class:`.ErrorBarCurveItem` object

        Example::
            error(x, y, None, dy, marker='Ellipse', markerfacecolor='#ffffff')
        which is equivalent to (MATLAB-style support)::
            error(x, y, None, dy, marker='o', markerfacecolor='w')
        """
        basename = _("Curve")
        curveparam = CurveParam(title=basename, icon="curve.png")
        errorbarparam = ErrorBarParam(title=_("Error bars"), icon="errorbar.png")
        if not title:
            global CURVE_COUNT
            CURVE_COUNT += 1
            curveparam.label = make_title(basename, CURVE_COUNT)
        self.__set_param(
            curveparam,
            title,
            color,
            linestyle,
            linewidth,
            marker,
            markersize,
            markerfacecolor,
            markeredgecolor,
            shade,
            curvestyle,
            baseline,
            dsamp_factor,
            use_dsamp,
        )
        errorbarparam.color = curveparam.line.color
        if errorbarwidth is not None:
            errorbarparam.width = errorbarwidth
        if errorbarcap is not None:
            errorbarparam.cap = errorbarcap
        if errorbarmode is not None:
            errorbarparam.mode = errorbarmode
        if errorbaralpha is not None:
            errorbarparam.alpha = errorbaralpha
        return self.perror(x, y, dx, dy, curveparam, errorbarparam, xaxis, yaxis)

    def histogram(
        self,
        data: numpy.ndarray,
        bins: int | None = None,
        logscale: bool | None = None,
        title: str = "",
        color: str | None = None,
        xaxis: str = "bottom",
        yaxis: str = "left",
    ) -> HistogramItem:
        """Make 1D Histogram `plot item`

        Args:
            data: data
            bins: number of bins. Default is None
            logscale: Y-axis scale. Default is None
            title: curve title. Default is ''
            color: curve color name. Default is None
            xaxis: x axis name. Default is 'bottom'
            yaxis: y axis name. Default is 'left'

        Returns:
            :py:class:`.HistogramItem` object
        """
        basename = _("Histogram")
        histparam = HistogramParam(title=basename, icon="histogram.png")
        curveparam = CurveParam(_("Curve"), icon="curve.png")
        curveparam.read_config(CONF, "histogram", "curve")
        if not title:
            global HISTOGRAM_COUNT
            HISTOGRAM_COUNT += 1
            title = make_title(basename, HISTOGRAM_COUNT)
        curveparam.label = title
        if color is not None:
            curveparam.line.color = color
        if bins is not None:
            histparam.n_bins = bins
        if logscale is not None:
            histparam.logscale = logscale
        return self.phistogram(data, curveparam, histparam, xaxis, yaxis)

    def phistogram(
        self,
        data: numpy.ndarray,
        curveparam: CurveParam,
        histparam: HistogramParam,
        xaxis: str = "bottom",
        yaxis: str = "left",
    ) -> HistogramItem:
        """Make 1D histogram `plot item` based on a :py:class:`.CurveParam` and
        :py:class:`.HistogramParam` instances

        Args:
            data: data
            curveparam: curve parameters
            histparam: histogram parameters
            xaxis: x axis name. Default is 'bottom'
            yaxis: y axis name. Default is 'left'

        Returns:
            :py:class:`.HistogramItem` object

        Example::
            phistogram(data, curveparam, histparam)
        """
        hist = HistogramItem(curveparam, histparam)
        hist.update_params()
        hist.set_hist_data(data)
        self.__set_curve_axes(hist, xaxis, yaxis)
        return hist

    def range(self, xmin: float, xmax: float) -> XRangeSelection:
        """Make a range `plot item`

        Args:
            xmin: minimum value
            xmax: maximum value

        Returns:
            :py:class:`.XRangeSelection` object
        """
        return XRangeSelection(xmin, xmax)

    def vcursor(
        self,
        x: float,
        label: str | None = None,
        constraint_cb: Callable | None = None,
        movable: bool = True,
        readonly: bool = False,
    ) -> Marker:
        """Make a vertical cursor `plot item`

        Args:
            x: cursor position
            label: cursor label. Default is None
            constraint_cb: constraint callback. Default is None
            movable: enable/disable cursor move. Default is True
            readonly: enable/disable cursor edition. Default is False

        Returns:
            :py:class:`.Marker` object
        """
        if label is None:

            def label_cb(x, y):
                """Label callback"""
                return ""

        else:

            def label_cb(x, y):
                """Label callback"""
                return label % x

        return self.marker(
            position=(x, 0),
            markerstyle="|",
            label_cb=label_cb,
            constraint_cb=constraint_cb,
            movable=movable,
            readonly=readonly,
        )

    def hcursor(
        self,
        y: float,
        label: str | None = None,
        constraint_cb: Callable | None = None,
        movable: bool = True,
        readonly: bool = False,
    ) -> Marker:
        """Make an horizontal cursor `plot item`

        Args:
            y: cursor position
            label: cursor label. Default is None
            constraint_cb: constraint callback. Default is None
            movable: enable/disable cursor move. Default is True
            readonly: enable/disable cursor edition. Default is False

        Returns:
            :py:class:`.Marker` object
        """
        if label is None:

            def label_cb(x, y):
                """Label callback"""
                return ""

        else:

            def label_cb(x, y):
                """Label callback"""
                return label % y

        return self.marker(
            position=(0, y),
            markerstyle="-",
            label_cb=label_cb,
            constraint_cb=constraint_cb,
            movable=movable,
            readonly=readonly,
        )

    def xcursor(
        self,
        x: float,
        y: float,
        label: str | None = None,
        constraint_cb: Callable | None = None,
        movable: bool = True,
        readonly: bool = False,
    ) -> Marker:
        """Make a cross cursor `plot item`

        Args:
            x: cursor position
            y: cursor position
            label: cursor label. Default is None
            constraint_cb: constraint callback. Default is None
            movable: enable/disable cursor move. Default is True
            readonly: enable/disable cursor edition. Default is False

        Returns:
            :py:class:`.Marker` object
        """
        if label is None:

            def label_cb(x, y):
                """Label callback"""
                return ""

        else:

            def label_cb(x, y):
                """Label callback"""
                return label % (x, y)

        return self.marker(
            position=(x, y),
            markerstyle="+",
            label_cb=label_cb,
            constraint_cb=constraint_cb,
            movable=movable,
            readonly=readonly,
        )

    def marker(
        self,
        position: tuple[float, float] | None = None,
        label_cb: Callable | None = None,
        constraint_cb: Callable | None = None,
        movable: bool = True,
        readonly: bool = False,
        markerstyle: str | None = None,
        markerspacing: float | None = None,
        color: str | None = None,
        linestyle: str | None = None,
        linewidth: float | None = None,
        marker: str | None = None,
        markersize: float | None = None,
        markerfacecolor: str | None = None,
        markeredgecolor: str | None = None,
    ) -> Marker:
        """Make a marker `plot item`

        Args:
            position: marker position
            label_cb: label callback. Default is None
            constraint_cb: constraint callback. Default is None
            movable: enable/disable marker move. Default is True
            readonly: enable/disable marker edition. Default is False
            markerstyle: marker style. Default is None
            markerspacing: spacing between text and marker line.
             Default is None
            color: marker color name. Default is None
            linestyle: marker line style (MATLAB-like string or attribute
             name from the `PyQt5.QtCore.Qt.PenStyle` enum
             (i.e. "SolidLine" "DashLine", "DotLine", "DashDotLine",
             "DashDotDotLine" or "NoPen"). Default is None
            linewidth: line width (pixels). Default is None
            marker: marker shape (MATLAB-like string or "Cross", "Plus",
             "Circle", "Disc", "Square", "Diamond", "TriangleUp",
             "TriangleDown", "TriangleLeft", "TriangleRight", "TriRight",
             "TriLeft", "TriDown", "TriUp", "Octagon", "Star1", "Star2",
             "Pentagon", "Hexagon1", "Hexagon2", "Plus (filled)", "Cross
             (filled)", "Square (filled)", "Diamond (filled)", "TriangleUp
             (filled)", "TriangleDown (filled)", "TriangleLeft (filled)",
             "TriangleRight (filled)", "TriRight (filled)", "TriLeft
             (filled)", "TriDown (filled)", "TriUp (filled)", "Octagon
             (filled)", "Star1 (filled)", "Star2 (filled)", "Pentagon
             (filled)", "Hexagon1 (filled)", "Hexagon2 (filled)"). Default is
             None
            markersize: marker size (pixels). Default is None
            markerfacecolor: marker face color name. Default is None
            markeredgecolor: marker edge color name. Default is None

        Returns:
            :py:class:`.Marker` object
        """
        param = MarkerParam(_("Marker"), icon="marker.png")
        param.read_config(CONF, "plot", "marker/cursor")
        if (
            color
            or linestyle
            or linewidth
            or marker
            or markersize
            or markerfacecolor
            or markeredgecolor
        ):
            param.line = param.sel_line
            param.symbol = param.sel_symbol
            param.text = param.sel_text
            self.__set_baseparam(
                param,
                color,
                linestyle,
                linewidth,
                marker,
                markersize,
                markerfacecolor,
                markeredgecolor,
            )
            param.sel_line = param.line
            param.sel_symbol = param.symbol
            param.sel_text = param.text
        if markerstyle:
            param.set_markerstyle(markerstyle)
        if markerspacing:
            param.spacing = markerspacing
        if not movable:
            param.symbol.marker = param.sel_symbol.marker = "NoSymbol"
        marker = Marker(
            label_cb=label_cb, constraint_cb=constraint_cb, markerparam=param
        )
        if position is not None:
            x, y = position
            marker.set_pos(x, y)
        marker.set_readonly(readonly)
        if not movable:
            marker.set_movable(False)
            marker.set_resizable(False)
        return marker



# -*- coding: utf-8 -*-

"""A custom QDoubleRangeSlider with some extra functionnalities for colormap sliders.
"""
from typing import Sequence

from plotpy.external.sliders import QDoubleRangeSlider


class QColorMapSlider(QDoubleRangeSlider):
    """Supercharged QDoubleRangeSlider with a few extra features for colormap
    sliders."""

    @property
    def pressed_index(self) -> int:
        """Simple getter for the index of the handle pressed by the user
        (mouse clicked).

        Returns:
            handled index
        """
        return self._pressedIndex

    def widget_pos_to_value(self, pos: int) -> float:
        """Returns the relative value of the slider at the given pixel position on the
        widget.

        Returns:
            slider value
        """
        return self._pixelPosToRangeValue(pos)

    def filter_first_last_positions(self, value: Sequence[float]) -> Sequence[float]:
        """Checks if the first and last handles are at the minimum and maximum values.
        If not, it sets them to the minimum and maximum values. This is required to
        prevent the application from crashing.

        Args:
            value: list of handle values

        Returns:
            Modified list of handle values if necessary
        """
        if value[0] != self._minimum or value[-1] != self._maximum:
            value = list(value)
            value[0], value[-1] = self._minimum, self._maximum
        return value

    def _setPosition(self, val: Sequence[float]) -> None:
        """Overload of the original _setPosition method to prevent the application from
        crashing if the first and last handles are moved.

        Args:
            value: list of handle values
        """
        return super()._setPosition(self.filter_first_last_positions(val))

    def setValue(self, value: Sequence[float]) -> None:
        """Overload of the original setValue method to prevent the application from
        crashing if the first and last handles are moved.

        Args:
            value: list of handle values
        """
        return super().setValue(self.filter_first_last_positions(value))



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

"""Test cyclic import issue"""

# pylint: disable=import-outside-toplevel
# pylint: disable=unused-import


def test_tools_cyclic_import():
    """Test cyclic import issue"""
    # Importing one tool from each module to check if there is a cyclic import issue
    from plotpy.tools import (
        AnnotatedPointTool,  # noqa: F401
        AverageCrossSectionTool,  # noqa: F401
        CurveStatsTool,  # noqa: F401
        DoAutoscaleTool,  # noqa: F401
        ItemCenterTool,  # noqa: F401
        OpenImageTool,  # noqa: F401
        PointTool,  # noqa: F401
        PrintTool,  # noqa: F401
        RectangularActionTool,  # noqa: F401
        SelectTool,  # noqa: F401
    )


if __name__ == "__main__":
    test_tools_cyclic_import()



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

"""Simple dialog box based on plotpy"""

# guitest: show

import scipy.ndimage
from guidata.dataset import ChoiceItem, DataSet, IntItem, StringItem, update_dataset
from guidata.dataset.qtwidgets import DataSetEditGroupBox, DataSetShowGroupBox
from guidata.qthelpers import qt_app_context

from plotpy import io
from plotpy.builder import make
from plotpy.config import _
from plotpy.plot import PlotDialog, PlotOptions
from plotpy.tools import OpenImageTool

# guitest: show


class ImageParam(DataSet):
    title = StringItem(_("Title"))
    width = IntItem(_("Width"), help=_("Image width (pixels)"))
    height = IntItem(_("Height"), help=_("Image height (pixels)"))


class FilterParam(DataSet):
    name = ChoiceItem(
        _("Filter algorithm"),
        (
            ("gaussian_filter", _("gaussian filter")),
            ("uniform_filter", _("uniform filter")),
            ("minimum_filter", _("minimum filter")),
            ("median_filter", _("median filter")),
            ("maximum_filter", _("maximum filter")),
        ),
    )
    size = IntItem(_("Size or sigma"), min=1, default=5)


class ExampleDialog(PlotDialog):
    def __init__(
        self,
        title=_("Example dialog box"),
        options=PlotOptions(show_contrast=True, type="image"),
        edit=False,
    ):
        self.filter_gbox = None
        self.data = None
        self.item = None
        options.title = _("Image title")
        options.zlabel = _("z-axis scale label")
        super().__init__(
            title=title,
            toolbar=True,
            edit=edit,
            options=options,
            auto_tools=False,
        )
        self.resize(600, 600)
        opentool = self.manager.add_tool(OpenImageTool)
        opentool.SIG_OPEN_FILE.connect(self.open_image)
        self.manager.register_all_image_tools()
        self.manager.activate_default_tool()

    def populate_plot_layout(self):
        """Populate the plot layout"""
        self.filter_gbox = DataSetEditGroupBox(_("Filter parameters"), FilterParam)
        self.filter_gbox.setEnabled(False)
        self.filter_gbox.SIG_APPLY_BUTTON_CLICKED.connect(self.apply_filter)
        self.add_widget(self.filter_gbox, 0, 0)
        self.param_gbox = DataSetShowGroupBox(_("Image parameters"), ImageParam)
        self.add_widget(self.param_gbox, 0, 1)
        self.add_widget(self.plot_widget, 1, 0, 1, 0)

    def open_image(self, filename):
        """Opening image *filename*"""
        self.data = io.imread(filename, to_grayscale=True)
        self.show_data(self.data)
        param = ImageParam()
        param.title = filename
        param.height, param.width = self.data.shape
        update_dataset(self.param_gbox.dataset, param)
        self.param_gbox.get()
        self.filter_gbox.setEnabled(True)

    def show_data(self, data):
        plot = self.manager.get_plot()
        if self.item is not None:
            self.item.set_data(data)
        else:
            self.item = make.image(data, colormap="gray")
            plot.add_item(self.item, z=0)
        plot.set_active_item(self.item)
        plot.replot()

    def apply_filter(self):
        param = self.filter_gbox.dataset
        filterfunc = getattr(scipy.ndimage, param.name)
        data = filterfunc(self.data, param.size)
        self.show_data(data)


def test_simple_dialog():
    """Test simple dialog"""
    with qt_app_context(exec_loop=True):
        dlg = ExampleDialog()
        dlg.show()


if __name__ == "__main__":
    test_simple_dialog()



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

"""DICOM image test

Requires pydicom (>=0.9.3)"""

# guitest: show

import pytest
from guidata.qthelpers import qt_app_context

from plotpy.builder import make
from plotpy.tests import get_path

try:
    import pydicom  # type:ignore
except ImportError:
    pydicom = None


@pytest.mark.skipif(pydicom is None, reason="pydicom not installed")
def test_dicom_image():
    with qt_app_context(exec_loop=True):
        win = make.dialog(
            edit=False,
            toolbar=True,
            wintitle="DICOM I/O test",
            show_contrast=True,
            type="image",
            size=(600, 700),
        )
        filename = get_path("mr-brain.dcm")
        image = make.image(filename=filename, title="DICOM img", colormap="gray")
        plot = win.manager.get_plot()
        plot.add_item(image)
        plot.select_item(image)
        contrast = win.manager.get_contrast_panel()
        contrast.histogram.eliminate_outliers(54.0)


if __name__ == "__main__":
    win = test_dicom_image()



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

"""
Masked Image test, creating the MaskedXYImageItem object via make.maskedxyimage

Masked image XY items are constructed using a masked array item. Masked data is
ignored in computations, like the average cross sections.
"""

# guitest: show

from __future__ import annotations

from plotpy import io
from plotpy.builder import make
from plotpy.tests.items.test_image_masked import MaskedImageTest


class XYMaskedImageTest(MaskedImageTest):
    """XYMaskedImageItem test class"""

    def build_items(self) -> list:
        """Build items"""
        data = io.imread(self.IMAGE_FN, to_grayscale=True)
        x = [0]
        delta = 1
        for x_val in range(data.shape[0] - 1):
            delta = delta + 0.1
            x.append(x[-1] + delta)
        y = [0]
        for y_val in range(data.shape[1] - 1):
            delta = delta + 0.1
            y.append(y[-1] + delta)
        image = make.maskedimage(data=data, colormap="gray", show_mask=True, x=x, y=y)
        return [image]


def test_image_masked_xy():
    """Test MaskedImageItem"""
    test = XYMaskedImageTest("MaskedImageItem test")
    test.run()


if __name__ == "__main__":
    test_image_masked_xy()



# -*- coding: utf-8 -*-

from __future__ import annotations

import sys
import weakref
from typing import TYPE_CHECKING

import numpy as np
from qtpy import QtCore as QC

from plotpy.config import _
from plotpy.constants import X_BOTTOM, Y_LEFT
from plotpy.coords import axes_to_canvas, canvas_to_axes
from plotpy.interfaces import IBasePlotItem
from plotpy.items.curve.errorbar import ErrorBarCurveItem
from plotpy.items.image.misc import get_image_from_qrect
from plotpy.mathutils.geometry import rotate, translate, vector_angle, vector_norm

if TYPE_CHECKING:
    from plotpy.items import AnnotatedObliqueRectangle, AnnotatedSegment

try:
    from plotpy._scaler import INTERP_LINEAR, _scale_tr
except ImportError:
    print(
        ("Module 'plotpy.items.image.base': missing C extension"),
        file=sys.stderr,
    )
    print(
        ("try running :" "python setup.py build_ext --inplace -c mingw32"),
        file=sys.stderr,
    )
    raise

TEMP_ITEM = None


def get_rectangular_area(obj):
    """
    Return rectangular area covered by object

    Return None if object does not support this feature
    (like markers, points, ...)
    """
    if hasattr(obj, "get_rect"):
        return obj.get_rect()


def get_object_coordinates(obj):
    """Return Marker or PointShape/AnnotatedPoint object coordinates"""
    if hasattr(obj, "get_pos"):
        return obj.get_pos()
    else:
        return obj.xValue(), obj.yValue()


def get_plot_x_section(obj, apply_lut=False):
    """
    Return plot cross section along x-axis,
    at the y value defined by 'obj', a Marker/AnnotatedPoint object
    """
    _x0, y0 = get_object_coordinates(obj)
    plot = obj.plot()
    xmap = plot.canvasMap(plot.X_BOTTOM)
    xc0, xc1 = xmap.p1(), xmap.p2()
    _xc0, yc0 = axes_to_canvas(obj, 0, y0)
    if plot.get_axis_direction("left"):
        yc1 = yc0 + 1
    else:
        yc1 = yc0 - 3
    try:
        # TODO: Eventually add an option to apply interpolation algorithm
        data = get_image_from_qrect(
            plot,
            QC.QPointF(xc0, yc0),
            QC.QPointF(xc1, yc1),
            apply_lut=apply_lut,
            add_images=True,
            apply_interpolation=False,
        )
    except (ValueError, ZeroDivisionError, TypeError):
        return np.array([]), np.array([])
    if data.size == 0:
        return np.array([]), np.array([])
    y = data.mean(axis=0)
    x0, _y0 = canvas_to_axes(obj, QC.QPointF(xc0, yc0))
    x1, _y1 = canvas_to_axes(obj, QC.QPointF(xc1, yc1))
    x = np.linspace(x0, x1, len(y))
    return x, y


def get_plot_y_section(obj, apply_lut=False):
    """
    Return plot cross section along y-axis,
    at the x value defined by 'obj', a Marker/AnnotatedPoint object
    """
    x0, _y0 = get_object_coordinates(obj)
    plot = obj.plot()
    ymap = plot.canvasMap(Y_LEFT)
    yc0, yc1 = ymap.p1(), ymap.p2()
    if plot.get_axis_direction("left"):
        yc1, yc0 = yc0, yc1
    xc0, _yc0 = axes_to_canvas(obj, x0, 0)
    xc1 = xc0 + 1
    try:
        data = get_image_from_qrect(
            plot,
            QC.QPointF(xc0, yc0),
            QC.QPointF(xc1, yc1),
            apply_lut=apply_lut,
            add_images=True,
            apply_interpolation=False,
        )
    except (ValueError, ZeroDivisionError, TypeError):
        return np.array([]), np.array([])
    if data.size == 0:
        return np.array([]), np.array([])
    y = data.mean(axis=1)
    _x0, y0 = canvas_to_axes(obj, QC.QPointF(xc0, yc0))
    _x1, y1 = canvas_to_axes(obj, QC.QPointF(xc1, yc1))
    x = np.linspace(y0, y1, len(y))
    return x, y


def get_plot_average_x_section(obj, apply_lut=False):
    """
    Return cross section along x-axis, averaged on ROI defined by 'obj'
    'obj' is an AbstractShape object supporting the 'get_rect' method
    (RectangleShape, AnnotatedRectangle, etc.)
    """
    x0, y0, x1, y1 = obj.get_rect()
    xc0, yc0 = axes_to_canvas(obj, x0, y0)
    xc1, yc1 = axes_to_canvas(obj, x1, y1)
    invert = False
    if xc0 > xc1:
        invert = True
        xc1, xc0 = xc0, xc1
    ydir = obj.plot().get_axis_direction("left")
    if (ydir and yc0 > yc1) or (not ydir and yc0 < yc1):
        yc1, yc0 = yc0, yc1
    try:
        data = get_image_from_qrect(
            obj.plot(),
            QC.QPointF(xc0, yc0),
            QC.QPointF(xc1, yc1),
            apply_lut=apply_lut,
            apply_interpolation=False,
        )
    except (ValueError, ZeroDivisionError, TypeError):
        return np.array([]), np.array([])
    if data.size == 0:
        return np.array([]), np.array([])
    y = data.mean(axis=0)
    if invert:
        y = y[::-1]
    x = np.linspace(x0, x1, len(y))
    return x, y


def get_plot_average_y_section(obj, apply_lut=False):
    """
    Return cross section along y-axis, averaged on ROI defined by 'obj'
    'obj' is an AbstractShape object supporting the 'get_rect' method
    (RectangleShape, AnnotatedRectangle, etc.)
    """
    x0, y0, x1, y1 = obj.get_rect()
    xc0, yc0 = axes_to_canvas(obj, x0, y0)
    xc1, yc1 = axes_to_canvas(obj, x1, y1)
    invert = False
    ydir = obj.plot().get_axis_direction("left")
    if (ydir and yc0 > yc1) or (not ydir and yc0 < yc1):
        invert = True
        yc1, yc0 = yc0, yc1
    if xc0 > xc1:
        xc1, xc0 = xc0, xc1
    try:
        data = get_image_from_qrect(
            obj.plot(),
            QC.QPointF(xc0, yc0),
            QC.QPointF(xc1, yc1),
            apply_lut=apply_lut,
            apply_interpolation=False,
        )
    except (ValueError, ZeroDivisionError, TypeError):
        return np.array([]), np.array([])
    if data.size == 0:
        return np.array([]), np.array([])
    y = data.mean(axis=1)
    x = np.linspace(y0, y1, len(y))
    if invert:
        x = x[::-1]
    return x, y


def compute_oblique_section(item, obj, debug=False):
    """Return oblique averaged cross section"""
    global TEMP_ITEM

    if obj.plot() is None:
        # Item has not yet been added to the plot
        return np.array([]), np.array([])

    xa, ya, xb, yb = obj.get_bounding_rect_coords()
    x0, y0, x1, y1, x2, y2, x3, y3 = obj.get_rect()

    getcpi = item.get_closest_pixel_indexes
    ixa, iya = getcpi(xa, ya)
    ixb, iyb = getcpi(xb, yb)
    ix0, iy0 = getcpi(x0, y0)
    ix1, iy1 = getcpi(x1, y1)
    ix3, iy3 = getcpi(x3, y3)

    destw = int(vector_norm(ix0, iy0, ix1, iy1))
    desth = int(vector_norm(ix0, iy0, ix3, iy3))
    ysign = -1 if obj.plot().get_axis_direction("left") else 1
    angle = vector_angle(ix1 - ix0, (iy1 - iy0) * ysign)

    dst_rect = (0, 0, int(destw), int(desth))
    dst_image = np.empty((int(desth), int(destw)), dtype=np.float64)

    if isinstance(item.data, np.ma.MaskedArray):
        if item.data.dtype in (np.float32, np.float64):
            item_data = item.data
        else:
            item_data = np.ma.array(item.data, dtype=np.float32, copy=True)
        data = np.ma.filled(item_data, np.nan)
    else:
        data = item.data

    ixr = 0.5 * (ixb + ixa)
    iyr = 0.5 * (iyb + iya)
    mat = translate(ixr, iyr) @ rotate(-angle) @ translate(-0.5 * destw, -0.5 * desth)
    _scale_tr(data, mat, dst_image, dst_rect, (1.0, 0.0, np.nan), (INTERP_LINEAR,))

    if debug:
        plot = obj.plot()
        if TEMP_ITEM is None:
            from plotpy.builder import make

            TEMP_ITEM = make.image(dst_image)
            plot.add_item(TEMP_ITEM)
        else:
            TEMP_ITEM.set_data(dst_image)
        if False:
            from plotpy.constants import LUTAlpha

            TEMP_ITEM.param.alpha_function = LUTAlpha.LINEAR.value
            xmin, ymin = ixa, iya
            xmax, ymax = xmin + destw, ymin + desth
            TEMP_ITEM.param.xmin = xmin
            TEMP_ITEM.param.xmax = xmax
            TEMP_ITEM.param.ymin = ymin
            TEMP_ITEM.param.ymax = ymax
            TEMP_ITEM.param.update_item(TEMP_ITEM)
        plot.replot()

    fixed_image = np.ma.fix_invalid(dst_image, copy=debug)
    if fixed_image.size == 0:
        return np.array([]), np.array([])
    ydata = fixed_image.mean(axis=1)
    xdata = item.get_x_values(0, ydata.size)[: ydata.size]
    try:
        xdata -= xdata[0]
    except IndexError:
        print(xdata, ydata)
    return xdata, ydata


class CrossSectionItem(ErrorBarCurveItem):
    """A Qwt item representing cross section data"""

    __implements__ = (IBasePlotItem,)
    ORIENTATION = None

    def __init__(self, curveparam=None, errorbarparam=None):
        ErrorBarCurveItem.__init__(self, curveparam, errorbarparam)
        self.setOrientation(self.ORIENTATION)
        self.perimage_mode = True
        self.autoscale_mode = False
        self.apply_lut = False
        self.source = None

    def set_source_image(self, src):
        """
        Set source image
        (source: object with methods 'get_xsection' and 'get_ysection',
         e.g. objects derived from plotpy.items.image.BaseImageItem)
        """
        self.source = weakref.ref(src)

    def get_source_image(self):
        """

        :return:
        """
        if self.source is not None:
            return self.source()

    def get_cross_section(self, obj):
        """Get cross section data from source image"""
        raise NotImplementedError

    def clear_data(self):
        """ """
        self.set_data(np.array([]), np.array([]), None, None)
        self.plot().SIG_CS_CURVE_CHANGED.emit(self)

    def setStyle(self, style):
        """
        Update the curve style and update the curve data to shift axes
        according to the style.
        """
        super().setStyle(style)
        plot = self.plot()
        if plot is not None:
            obj = plot.get_last_obj()
            if obj is not None:
                self.update_curve_data(obj)

    def update_curve_data(self, obj):
        """

        :param obj:
        """
        sectx, secty = self.get_cross_section(obj)
        if secty.size == 0 or np.all(np.isnan(secty)):
            sectx, secty = np.array([]), np.array([])
        elif self.param.curvestyle != "Steps" and sectx.size > 1:
            # Center the symbols at the middle of pixels:
            sectx[:-1] += np.mean(np.diff(sectx) / 2)
        if self.orientation() == QC.Qt.Orientation.Vertical:
            self.process_curve_data(secty, sectx)
        else:
            self.process_curve_data(sectx, secty)

    def process_curve_data(self, x, y, dx=None, dy=None):
        """
        Override this method to process data
        before updating the displayed curve
        """
        self.set_data(x, y, dx, dy)

    def update_item(self, obj):
        """

        :param obj:
        :return:
        """
        plot = self.plot()
        if not plot:
            return
        source = self.get_source_image()
        if source is None or not plot.isVisible():
            return
        self.update_curve_data(obj)
        self.plot().SIG_CS_CURVE_CHANGED.emit(self)
        if not self.autoscale_mode:
            self.update_scale()

    def update_scale(self):
        """ """
        plot = self.plot()
        if self.orientation() == QC.Qt.Orientation.Vertical:
            axis_id = Y_LEFT
        else:
            axis_id = X_BOTTOM
        source = self.get_source_image()
        sdiv = source.plot().axisScaleDiv(axis_id)
        plot.setAxisScale(axis_id, sdiv.lowerBound(), sdiv.upperBound())
        plot.replot()


class XCrossSectionItem(CrossSectionItem):
    """A Qwt item representing x-axis cross section data"""

    ORIENTATION = QC.Qt.Orientation.Horizontal

    def get_cross_section(self, obj):
        """Get x-cross section data from source image"""
        source = self.get_source_image()
        rect = get_rectangular_area(obj)
        fmt = source.param.yformat
        if rect is None:
            # Object is a marker or an annotated point
            _x0, y0 = get_object_coordinates(obj)
            self.param.label = _("Cross section") + " @ y=" + (fmt % y0)
            if self.perimage_mode:
                data = source.get_xsection(y0, apply_lut=self.apply_lut)
            else:
                data = get_plot_x_section(obj, apply_lut=self.apply_lut)
        else:
            if self.perimage_mode:
                x0, y0, x1, y1 = rect
                data = source.get_average_xsection(
                    x0, y0, x1, y1, apply_lut=self.apply_lut
                )
            else:
                data = get_plot_average_x_section(obj, apply_lut=self.apply_lut)
                x0, y0, x1, y1 = obj.get_rect()
            text = _("Average cross section")
            self.param.label = f"{text} @ y=[{fmt % y0} ; {fmt % y1}]"
        return data


class YCrossSectionItem(CrossSectionItem):
    """A Qwt item representing y-axis cross section data"""

    ORIENTATION = QC.Qt.Orientation.Vertical

    def get_cross_section(self, obj):
        """Get y-cross section data from source image"""
        source = self.get_source_image()
        rect = get_rectangular_area(obj)
        fmt = source.param.xformat
        if rect is None:
            # Object is a marker or an annotated point
            x0, _y0 = get_object_coordinates(obj)
            self.param.label = _("Cross section") + " @ x=" + (fmt % x0)
            if self.perimage_mode:
                data = source.get_ysection(x0, apply_lut=self.apply_lut)
            else:
                data = get_plot_y_section(obj, apply_lut=self.apply_lut)
        else:
            if self.perimage_mode:
                x0, y0, x1, y1 = rect
                data = source.get_average_ysection(
                    x0, y0, x1, y1, apply_lut=self.apply_lut
                )
            else:
                data = get_plot_average_y_section(obj, apply_lut=self.apply_lut)
                x0, y0, x1, y1 = obj.get_rect()
            text = _("Average cross section")
            self.param.label = f"{text} @ x=[{fmt % x0} ; {fmt % x1}]"
        return data


# Oblique cross section item
class ObliqueCrossSectionItem(CrossSectionItem):
    """A Qwt item representing radially-averaged cross section data"""

    DEBUG = False

    def update_curve_data(self, obj: AnnotatedObliqueRectangle) -> None:
        """Update curve data"""
        source = self.get_source_image()
        rect = obj.get_bounding_rect_coords()
        if rect is not None and source.data is not None:
            #            x0, y0, x1, y1 = rect
            #            angle = obj.get_tr_angle()
            sectx, secty = compute_oblique_section(source, obj, debug=self.DEBUG)
            if secty.size == 0 or np.all(np.isnan(secty)):
                sectx, secty = np.array([]), np.array([])
            self.process_curve_data(sectx, secty, None, None)

    def update_scale(self):
        """ """
        pass


def compute_line_section(
    data: np.ndarray, row0, col0, row1, col1
) -> tuple[np.ndarray, np.ndarray]:
    """Return intensity profile of data along a line

    Args:
        data: 2D array
        row0, col0: start point
        row1, col1: end point
    """
    # Keep coordinates inside the image
    row0 = max(0, min(row0, data.shape[0] - 1))
    col0 = max(0, min(col0, data.shape[1] - 1))
    row1 = max(0, min(row1, data.shape[0] - 1))
    col1 = max(0, min(col1, data.shape[1] - 1))
    # Keep coordinates in the right order
    row0, row1 = min(row0, row1), max(row0, row1)
    col0, col1 = min(col0, col1), max(col0, col1)
    # Extract the line
    line = np.zeros((2, max(abs(row1 - row0), abs(col1 - col0)) + 1), dtype=int)
    line[0, :] = np.linspace(row0, row1, line.shape[1]).astype(int)
    line[1, :] = np.linspace(col0, col1, line.shape[1]).astype(int)
    # Interpolate the line
    y = np.ma.array(data[line[0], line[1]], float).filled(np.nan)
    x = np.arange(y.size)
    return x, y


# Line cross section item
class LineCrossSectionItem(CrossSectionItem):
    """A Qwt item representing line cross section data"""

    def update_curve_data(self, obj: AnnotatedSegment) -> None:
        """Update curve data"""
        source = self.get_source_image()
        rect = obj.get_rect()
        if rect is not None and source.data is not None:
            x0, y0, x1, y1 = obj.get_rect()
            c0, r0 = source.get_closest_pixel_indexes(x0, y0)
            c1, r1 = source.get_closest_pixel_indexes(x1, y1)
            sectx, secty = compute_line_section(source.data, r0, c0, r1, c1)
            if secty.size == 0 or np.all(np.isnan(secty)):
                sectx, secty = np.array([]), np.array([])
            self.process_curve_data(sectx, secty, None, None)

    def update_scale(self):
        """ """
        pass



from __future__ import annotations

import numpy as np
from guidata.qthelpers import exec_dialog, qt_app_context

from plotpy.interfaces.items import IImageItemType
from plotpy.panels.csection.cswidget import LineCrossSection
from plotpy.tests.unit.utils import create_window, drag_mouse
from plotpy.tools import LineCrossSectionTool


def test_line_cross_section():
    """Test the line cross section tool."""
    with qt_app_context(exec_loop=False):
        win, _tool = create_window(
            LineCrossSectionTool,
            active_item_type=IImageItemType,
            panels=[LineCrossSection],
        )
        n = 100
        drag_mouse(win, np.linspace(0.25, 0.75, n), np.linspace(0.25, 0.75, n))
        exec_dialog(win)


if __name__ == "__main__":
    test_line_cross_section()



# -*- coding: utf-8 -*-
#
# For licensing and distribution details, please read carefully xgrid/__init__.py

# pylint: disable=invalid-name  # Allows short reference names like x, y, ...

from __future__ import annotations

from qtpy import QtCore as QC
from qtpy import QtGui as QG
from qtpy import QtSvg as QS
from qwt.scale_map import QwtScaleMap
from qwt.symbol import QwtSymbol

from plotpy.items import shape
from plotpy.styles import ShapeParam


class RectangleSVGShape(shape.RectangleShape):
    """Rectangle SVG shape

    Args:
        x1: X coordinate of the top-left corner
        y1: Y coordinate of the top-left corner
        x2: X coordinate of the bottom-right corner
        y2: Y coordinate of the bottom-right corner
        svg_data: SVG bytes
        shapeparam: Shape parameters
    """

    def __init__(
        self,
        x1: float = 0.0,
        y1: float = 0.0,
        x2: float = 0.0,
        y2: float = 0.0,
        svg_data: bytes | None = None,
        shapeparam: ShapeParam = None,
    ) -> None:
        self.svg_data = svg_data
        shape.RectangleShape.__init__(self, x1, y1, x2, y2, shapeparam)

    def set_data(self, svg_data: bytes) -> None:
        """Set SVG data"""
        self.svg_data = svg_data

    def draw(
        self,
        painter: QG.QPainter,
        xMap: QwtScaleMap,
        yMap: QwtScaleMap,
        canvasRect: QC.QRectF,
    ) -> None:
        """Draw shape (reimplement shape.Shape.draw))"""
        points = self.transform_points(xMap, yMap)
        renderer = QS.QSvgRenderer(self.svg_data)
        renderer.render(painter, points.boundingRect())


class SquareSVGShape(RectangleSVGShape):
    """Square SVG shape

    Args:
        svg_data: SVG bytes
        x1: X coordinate of the top-left corner
        y1: Y coordinate of the top-left corner
        x2: X coordinate of the bottom-right corner
        y2: Y coordinate of the bottom-right corner
        shapeparam: Shape parameters
    """

    def move_point_to(self, handle: int, pos: tuple[float], ctrl: bool = None):
        """Reimplement RectangleShape.move_point_to"""
        nx, ny = pos
        x1, y1, x2, y2 = self.get_rect()
        if handle == 0:
            self.set_rect(nx, y2 - (x2 - nx), x2, y2)
        elif handle == 1:
            self.set_rect(x1, y2 - (nx - x1), nx, y2)
        elif handle == 2:
            self.set_rect(x1, y1, nx, y1 + (nx - x1))
        elif handle == 3:
            self.set_rect(nx, y1, x2, y1 + (x2 - nx))
        elif handle == -1:
            delta = (nx, ny) - self.points.mean(axis=0)
            self.points += delta


class CircleSVGShape(shape.EllipseShape):
    """Circle SVG shape

    Args:
        x1: X coordinate of the top-left corner
        y1: Y coordinate of the top-left corner
        x2: X coordinate of the bottom-right corner
        y2: Y coordinate of the bottom-right corner
        svg_data: SVG bytes
        shapeparam: Shape parameters
    """

    def __init__(
        self,
        x1: float = 0.0,
        y1: float = 0.0,
        x2: float = 0.0,
        y2: float = 0.0,
        svg_data: bytes | None = None,
        shapeparam: ShapeParam = None,
    ) -> None:
        self.svg_data = svg_data
        shape.EllipseShape.__init__(self, x1, y1, x2, y2, shapeparam)

    def set_data(self, svg_data: bytes) -> None:
        """Set SVG data"""
        self.svg_data = svg_data

    def draw(
        self,
        painter: QG.QPainter,
        xMap: QwtScaleMap,
        yMap: QwtScaleMap,
        canvasRect: QC.QRect,
    ) -> None:
        """Draw shape (reimplement shape.Shape.draw))"""
        points, line0, line1, rect = self.compute_elements(xMap, yMap)
        if canvasRect.intersects(rect.toRect()) and self.svg_data is not None:
            pen, brush, symbol = self.get_pen_brush(xMap, yMap)
            painter.setRenderHint(QG.QPainter.Antialiasing)
            painter.setPen(pen)
            painter.setBrush(brush)
            painter.drawLine(line0)
            painter.drawLine(line1)
            painter.save()
            painter.translate(rect.center())
            painter.rotate(-line0.angle())
            painter.translate(-rect.center())
            renderer = QS.QSvgRenderer(self.svg_data)
            renderer.render(painter, rect)
            painter.restore()
            if symbol != QwtSymbol.NoSymbol:
                for i in range(points.size()):
                    symbol.drawSymbol(painter, points[i].toPoint())



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

"""Testing BasePlot API"""

# guitest: show

import os

from guidata.qthelpers import qt_app_context
from qtpy import QtCore as QC
from qtpy import QtWidgets as QW

from plotpy.tests import vistools as ptv
from plotpy.tests.features.test_auto_curve_image import make_curve_image_legend
from plotpy.tools.curve import EditPointTool, SelectPointsTool


def test_baseplot_api():
    """Testing BasePlot API"""
    with qt_app_context(exec_loop=True):
        items = make_curve_image_legend()
        win = ptv.show_items(items, wintitle=test_baseplot_api.__doc__)
        plot = win.get_plot()
        plot.manager.add_tool(SelectPointsTool)
        plot.manager.add_tool(EditPointTool)
        plot.get_default_item()
        title = "Test title"
        plot.set_title(title)
        assert plot.get_title() == title
        unit = "Test unit"
        plot.set_axis_unit("left", unit)
        assert plot.get_axis_unit("left") == unit
        plot.set_axis_ticks("left", 10, 10)
        plot.set_scales("lin", "lin")
        plot.enable_used_axes()
        plot.disable_unused_axes()
        plot.copy_to_clipboard()
        QW.QApplication.processEvents()
        fname = f"{test_baseplot_api.__name__}.pdf"
        plot.save_widget(fname)
        os.remove(fname)
        plot.hide_items(items)
        plot.show_items(items)
        plot.select_all()
        plot.move_up([items[0]])
        plot.move_down([items[0]])
        plot.unselect_item(items[0])
        plot.select_some_items(items)
        plot.unselect_all()
        plot.get_nearest_object(QC.QPointF(0, 0))
        plot.get_nearest_object_in_z(QC.QPointF(0, 0))
        plot.get_context_menu()
        plot.select_all()
        plot.edit_plot_parameters("item")
        plot.edit_axis_parameters(plot.yLeft)
        plot.set_titles(
            "Test title", "Test x title", "Test y title", "Test x unit", "Test y unit"
        )
        plot.notify_colormap_changed()


if __name__ == "__main__":
    test_baseplot_api()



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

"""All image and plot tools test"""

# guitest: show

from guidata.qthelpers import qt_app_context
from qtpy import QtWidgets as QW

from plotpy.builder import make
from plotpy.items import Marker
from plotpy.tests import get_path
from plotpy.tools import (
    AnnotatedCircleTool,
    AnnotatedEllipseTool,
    AnnotatedObliqueRectangleTool,
    AnnotatedPointTool,
    AnnotatedPolygonTool,
    AnnotatedRectangleTool,
    AnnotatedSegmentTool,
    CircleTool,
    EllipseTool,
    HCursorTool,
    HRangeTool,
    LabelTool,
    MultiLineTool,
    ObliqueRectangleTool,
    PlaceAxesTool,
    PolygonTool,
    RectangleTool,
    SegmentTool,
    VCursorTool,
    XCursorTool,
)

TOOLBAR_ID = "toolbar2"


def create_window():
    win = make.dialog(
        edit=False,
        toolbar=True,
        wintitle="All image and plot tools test",
        type="image",
        size=(800, 600),
    )
    toolbar2 = QW.QToolBar()
    win.layout().addWidget(toolbar2)
    win.manager.add_toolbar(toolbar2, TOOLBAR_ID)

    for toolklass in (
        LabelTool,
        HRangeTool,
        VCursorTool,
        HCursorTool,
        XCursorTool,
        SegmentTool,
        RectangleTool,
        ObliqueRectangleTool,
        CircleTool,
        EllipseTool,
        MultiLineTool,
        PolygonTool,
        PlaceAxesTool,
        AnnotatedRectangleTool,
        AnnotatedObliqueRectangleTool,
        AnnotatedCircleTool,
        AnnotatedEllipseTool,
        AnnotatedSegmentTool,
        AnnotatedPointTool,
        AnnotatedPolygonTool,
    ):
        win.manager.add_tool(toolklass, toolbar_id=TOOLBAR_ID)
    return win


def test_image_plot_tools():
    """Test"""
    with qt_app_context(exec_loop=True):
        filename = get_path("brain.png")
        win = create_window()
        win.show()
        image = make.image(filename=filename, colormap="bone")
        plot = win.manager.get_plot()
        plot.add_item(image)
        title = "toto"
        marker1 = Marker(label_cb=lambda x, y: f"{title}x = {x:g}<br>y = {y:g}")
        plot.add_item(marker1)
        marker2 = Marker(label_cb=lambda x, y: f"{title}x = {x:g}<br>y = {y:g}")
        plot.add_item(marker2)


if __name__ == "__main__":
    test_image_plot_tools()



from __future__ import annotations

import time
from typing import TYPE_CHECKING, TypeVar, Union

import qtpy.QtCore as QC
import qtpy.QtGui as QG
import qtpy.QtWidgets as QW
from qwt import QwtPlotItem

from plotpy.interfaces.items import ICurveItemType, IItemType
from plotpy.plot.base import BasePlot
from plotpy.tests import vistools as ptv
from plotpy.tests.features.test_auto_curve_image import make_curve_image_legend
from plotpy.tools import CommandTool, InteractiveTool

if TYPE_CHECKING:
    import numpy as np

    from plotpy.panels.base import PanelWidget
    from plotpy.plot.plotwidget import PlotDialog, PlotWindow


CLICK = (QC.QEvent.Type.MouseButtonPress, QC.QEvent.Type.MouseButtonRelease)
ToolT = TypeVar("ToolT", bound=Union[InteractiveTool, CommandTool])


def rel_pos_to_canvas_pos(
    canvas: BasePlot, relative_xy: tuple[float, float]
) -> tuple[QC.QPointF, QC.QPointF]:
    """Computes and return the relative position on the canvas and the corresponding
    abolute position in the window.

    Args:
        canvas: Plot canvas
        relative_xy: Relative position on the canvas (0 < xy <1)

    Returns:
        Tuple of the relative position on the canvas and the corresponding absolute
         position in the window.
    """
    size = canvas.size()
    pos_x, pos_y = (
        relative_xy[0] * size.width(),
        relative_xy[1] * size.height(),
    )
    canvas_pos = QC.QPointF(pos_x, pos_y)
    return canvas_pos, QC.QPointF(canvas.mapToGlobal(canvas_pos.toPoint()))


def keyboard_event(
    win: PlotDialog | PlotWindow,
    key: QC.Qt.Key,
    mod=QC.Qt.KeyboardModifier.NoModifier,
):
    """Simulates a keyboard event on the plot.

    Args:
        win: window containing the plot
        key: Key to simulate
        mod: Keyboard modifier. Defaults to QC.Qt.KeyboardModifier.NoModifier.

    Returns:
        None
    """
    plot = win.manager.get_plot()
    canva: QW.QWidget = plot.canvas()  # type: ignore
    key_event = QG.QKeyEvent(QC.QEvent.Type.KeyPress, key, mod)
    qapp = QW.QApplication.instance()
    qapp.sendEvent(canva, key_event)


def undo(win: PlotWindow):
    keyboard_event(win, QC.Qt.Key.Key_Z, mod=QC.Qt.KeyboardModifier.ControlModifier)


def redo(win: PlotWindow):
    keyboard_event(win, QC.Qt.Key.Key_Y, mod=QC.Qt.KeyboardModifier.ControlModifier)


def undo_redo(win: PlotWindow):
    undo(win)
    time.sleep(0.1)
    redo(win)


def new_wheel_event(
    canvas: BasePlot,
    pos_xy: tuple[float, float],
    angle_delta: int,
    pix_delta: int,
    btns: QC.Qt.MouseButton = QC.Qt.MouseButton.NoButton,
    mods=QC.Qt.KeyboardModifier.NoModifier,
) -> QG.QWheelEvent:
    canva_pos, glob_pos = rel_pos_to_canvas_pos(canvas, pos_xy)
    angle_delta = QC.QPoint(0, angle_delta)  # Scroll distance in eighths of a degree
    pix_delta = QC.QPoint(0, pix_delta)  # Scroll distance in pixels
    phase = QC.Qt.ScrollPhase.NoScrollPhase
    inverted = False

    # Create QWheelEvent
    return QG.QWheelEvent(
        canva_pos, glob_pos, pix_delta, angle_delta, btns, mods, phase, inverted
    )


def scroll_wheel(
    win: PlotDialog | PlotWindow,
    relative_xy: tuple[float, float],
    angle_delta: int,
    pix_delta: int = 0,
    btns: QC.Qt.MouseButton = QC.Qt.MouseButton.NoButton,
    mods=QC.Qt.KeyboardModifier.NoModifier,
) -> None:
    """Simulates a scroll wheel event on the plot at the given relative position.

    Args:
        win: window containing the plot
        relative_xy: Relative position of the click on the plot
        angle_delta: Scroll distance in eighths of a degree
        pix_delta: Scroll distance in pixels. Defaults to 0.
        btns: Mouse buttons. Defaults to QC.Qt.MouseButton.NoButton.
        mod: Keyboard modifier. Defaults to QC.Qt.KeyboardModifier.NoModifier.

    Returns:
        None
    """
    plot = win.manager.get_plot()
    canvas: BasePlot = plot.canvas()  # type: ignore
    wheel_event = new_wheel_event(
        canvas, relative_xy, angle_delta, pix_delta, btns, mods
    )
    qapp = QW.QApplication.instance()
    qapp.sendEvent(canvas, wheel_event)


def mouse_event_at_relative_plot_pos(
    win: PlotDialog | PlotWindow,
    relative_xy: tuple[float, float],
    click_types: tuple[QC.QEvent.Type, ...] = (QC.QEvent.Type.MouseButtonPress,),
    mod=QC.Qt.KeyboardModifier.NoModifier,
    btn=QC.Qt.MouseButton.LeftButton,
) -> None:
    """Simulates a click on the plot at the given relative position.

    Args:
        win: window containing the plot
        relative_xy: Relative position of the click on the plot
        click_types: Different mouse button event types to send.
         Defaults to (QC.QEvent.Type.MouseButtonPress,).
        mod: Keyboard modifier. Defaults to QC.Qt.KeyboardModifier.NoModifier.

    Returns:
        None
    """
    plot = win.manager.get_plot()
    canvas = plot.canvas()
    canvas_pos, _ = rel_pos_to_canvas_pos(canvas, relative_xy)

    qapp = QW.QApplication.instance()
    for type_ in click_types:
        mouse_event = QG.QMouseEvent(
            type_,
            canvas_pos,
            QC.QPointF(canvas.mapToGlobal(canvas_pos.toPoint())),
            btn,
            btn,
            mod,
        )
        qapp.sendEvent(canvas, mouse_event)


def drag_mouse(
    win: PlotWindow,
    x_path: np.ndarray,
    y_path: np.ndarray,
    mod=QC.Qt.KeyboardModifier.NoModifier,
    btn=QC.Qt.MouseButton.LeftButton,
    click=True,
) -> None:
    """Simulates a mouse drag on the plot.

    Args:
        win: window containing the plot
        x_path: relative x plot coordinates of the path to simulate the mouse drag along
        y_path: relative y plot coordinates of the path to simulate the mouse drag along
        mod: keyboard modifier. Defaults to QC.Qt.KeyboardModifier.NoModifier.
        click: Whether a click must be simulated (mouse hold -> drag -> mouse release).
         Defaults to True.
    """
    x0, y0 = x_path[0], y_path[0]
    xn, yn = x_path[-1], y_path[-1]
    press = (QC.QEvent.Type.MouseButtonPress,)
    move = (QC.QEvent.Type.MouseMove,)
    release = (QC.QEvent.Type.MouseButtonRelease,)

    if click:
        mouse_event_at_relative_plot_pos(win, (x0, y0), press, mod, btn)
    for x, y in zip(x_path, y_path):
        mouse_event_at_relative_plot_pos(win, (x, y), move, mod, btn)
    if click:
        mouse_event_at_relative_plot_pos(win, (xn, yn), release, mod, btn)


def create_window(
    tool_class: type[ToolT],
    active_item_type: type[IItemType] | None = ICurveItemType,
    panels: list[type[PanelWidget]] | None = None,
    items: list[QwtPlotItem] | None = None,
) -> tuple[PlotWindow, ToolT]:
    """Create a window with the given tool. The plot contains a curve, an image and a
     legend.

    Args:
        tool_class: tool class to add to the window.
        active_item_type: Type of Item to set to active (curve or image).
         Defaults to ICurveItemType.
        panels: PanelWidget classes to add. Defaults to None.

    Returns:
        The window containing the activated tool and the tool instance itself.
    """
    if items is None:
        items = make_curve_image_legend()
    win = ptv.show_items(items, wintitle="Unit test plot", auto_tools=False)
    plot = win.manager.get_plot()
    for item in items:
        plot.set_active_item(item)
    if active_item_type is None:
        plot.unselect_all()
        last_active_item = None
    else:
        last_active_item = plot.get_last_active_item(active_item_type)
    if last_active_item is not None:
        plot.set_active_item(last_active_item)

    if panels is not None:
        for panel in panels:
            win.manager.add_panel(panel())

    tool = win.manager.add_tool(tool_class)
    tool.activate()
    return win, tool



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

"""Resize test: using the scaler C++ engine to resize images"""

# guitest: show

from guidata.qthelpers import qt_app_context

from plotpy import io
from plotpy.mathutils import scaler
from plotpy.tests import get_path
from plotpy.tests.widgets.test_rotatecrop import imshow


def test_resize():
    """Test"""
    with qt_app_context(exec_loop=False):
        filename = get_path("brain.png")
        data = io.imread(filename)
        dst_image = scaler.resize(data, (2000, 3000))
        imshow(dst_image)


if __name__ == "__main__":
    test_resize()



# -*- coding: utf-8 -*-

from __future__ import annotations

from typing import TYPE_CHECKING

import numpy as np
from guidata.configtools import get_icon
from guidata.utils.misc import assert_interfaces_valid

from plotpy.items.shape.polygon import PolygonShape
from plotpy.mathutils.geometry import (
    compute_center,
    vector_norm,
    vector_projection,
    vector_rotation,
)

if TYPE_CHECKING:
    from plotpy.styles.shape import ShapeParam


def _no_null_vector(x0, y0, x1, y1, x2, y2, x3, y3):
    return (
        vector_norm(x0, y0, x1, y1)
        and vector_norm(x0, y0, x2, y2)
        and vector_norm(x0, y0, x3, y3)
        and vector_norm(x1, y1, x2, y2)
        and vector_norm(x1, y1, x3, y3)
        and vector_norm(x2, y2, x3, y3)
    )


class RectangleShape(PolygonShape):
    """Rectangle shape

    Args:
        x1: X coordinate of the top-left corner
        y1: Y coordinate of the top-left corner
        x2: X coordinate of the bottom-right corner
        y2: Y coordinate of the bottom-right corner
        shapeparam: Shape parameters
    """

    CLOSED = True

    def __init__(
        self,
        x1: float = 0.0,
        y1: float = 0.0,
        x2: float = 0.0,
        y2: float = 0.0,
        shapeparam: ShapeParam | None = None,
    ) -> None:
        super().__init__(shapeparam=shapeparam)
        self.set_rect(x1, y1, x2, y2)
        self.setIcon(get_icon("rectangle.png"))

    def set_rect(self, x1: float, y1: float, x2: float, y2: float) -> None:
        """Set the coordinates of the rectangle

        Args:
            x1: X coordinate of the top-left corner
            y1: Y coordinate of the top-left corner
            x2: X coordinate of the bottom-right corner
            y2: Y coordinate of the bottom-right corner
        """
        self.set_points([(x1, y1), (x2, y1), (x2, y2), (x1, y2)])

    def get_rect(self) -> tuple[float, float, float, float]:
        """Return the coordinates of the rectangle

        Returns:
            Coordinates of the rectangle
        """
        return tuple(self.points[0]) + tuple(self.points[2])

    def get_center(self) -> tuple[float, float]:
        """Return center coordinates

        Returns:
            Center coordinates
        """
        return compute_center(*self.get_rect())

    def move_point_to(
        self, handle: int, pos: tuple[float, float], ctrl: bool = False
    ) -> None:
        """Move a handle as returned by hit_test to the new position

        Args:
            handle: Handle
            pos: Position
            ctrl: True if <Ctrl> button is being pressed, False otherwise
        """
        nx, ny = pos
        x1, y1, x2, y2 = self.get_rect()
        if handle == 0:
            self.set_rect(nx, ny, x2, y2)
        elif handle == 1:
            self.set_rect(x1, ny, nx, y2)
        elif handle == 2:
            self.set_rect(x1, y1, nx, ny)
        elif handle == 3:
            self.set_rect(nx, y1, x2, ny)
        elif handle == -1:
            delta = (nx, ny) - self.points.mean(axis=0)
            self.points += delta

    def __reduce__(self) -> tuple:
        """Return the state of the object for pickling"""
        state = (self.shapeparam, self.points, self.z())
        return (self.__class__, (), state)

    def __setstate__(self, state: tuple) -> None:
        """Set the state of the object from pickling"""
        self.shapeparam, self.points, z = state
        self.setZ(z)
        self.shapeparam.update_item(self)


assert_interfaces_valid(RectangleShape)


class ObliqueRectangleShape(PolygonShape):
    """Oblique rectangle shape

    Args:
        x0: X coordinate of the top-left corner
        y0: Y coordinate of the top-left corner
        x1: X coordinate of the top-right corner
        y1: Y coordinate of the top-right corner
        x2: X coordinate of the bottom-right corner
        y2: Y coordinate of the bottom-right corner
        x3: X coordinate of the bottom-left corner
        y3: Y coordinate of the bottom-left corner
        shapeparam: Shape parameters
    """

    CLOSED = True
    ADDITIONNAL_POINTS = 2  # Number of points which are not part of the shape
    LINK_ADDITIONNAL_POINTS = True  # Link additionnal points with dotted lines

    def __init__(
        self,
        x0: float = 0.0,
        y0: float = 0.0,
        x1: float = 0.0,
        y1: float = 0.0,
        x2: float = 0.0,
        y2: float = 0.0,
        x3: float = 0.0,
        y3: float = 0.0,
        shapeparam: ShapeParam | None = None,
    ) -> None:
        super().__init__(shapeparam=shapeparam)
        self.set_rect(x0, y0, x1, y1, x2, y2, x3, y3)
        self.setIcon(get_icon("oblique_rectangle.png"))

    def set_rect(
        self,
        x0: float,
        y0: float,
        x1: float,
        y1: float,
        x2: float,
        y2: float,
        x3: float,
        y3: float,
    ) -> None:
        """Set the coordinates of the rectangle

        Args:
            x0: X coordinate of the top-left corner
            y0: Y coordinate of the top-left corner
            x1: X coordinate of the top-right corner
            y1: Y coordinate of the top-right corner
            x2: X coordinate of the bottom-right corner
            y2: Y coordinate of the bottom-right corner
            x3: X coordinate of the bottom-left corner
            y3: Y coordinate of the bottom-left corner

        ::

            x: additionnal points (handles used for rotation -- other handles
            being used for rectangle resizing)

            (x0, y0)------>(x1, y1)
                ↑             |
                |             |
                x             x
                |             |
                |             ↓
            (x3, y3)<------(x2, y2)
        """
        self.set_points(
            [
                (x0, y0),
                (x1, y1),
                (x2, y2),
                (x3, y3),
                (0.5 * (x0 + x3), 0.5 * (y0 + y3)),
                (0.5 * (x1 + x2), 0.5 * (y1 + y2)),
            ]
        )

    def get_rect(self) -> tuple[float, float, float, float, float, float, float, float]:
        """Return the coordinates of the rectangle

        Returns:
            Coordinates of the rectangle
        """
        return self.points.ravel()[: -self.ADDITIONNAL_POINTS * 2]

    def get_center(self) -> tuple[float, float]:
        """Return center coordinates

        Returns:
            Center coordinates
        """
        rect = tuple(self.points[0]) + tuple(self.points[2])
        return compute_center(*rect)

    def move_point_to(
        self, handle: int, pos: tuple[float, float], ctrl: bool = False
    ) -> None:
        """Move a handle as returned by hit_test to the new position

        Args:
            handle: Handle
            pos: Position
            ctrl: True if <Ctrl> button is being pressed, False otherwise
        """
        nx, ny = pos
        x0, y0, x1, y1, x2, y2, x3, y3 = self.get_rect()
        if handle == 0:
            if vector_norm(x2, y2, x3, y3) and vector_norm(x2, y2, x1, y1):
                v0n = np.array((nx - x0, ny - y0))
                x3, y3 = vector_projection(v0n, x2, y2, x3, y3)
                x1, y1 = vector_projection(v0n, x2, y2, x1, y1)
                x0, y0 = nx, ny
                if _no_null_vector(x0, y0, x1, y1, x2, y2, x3, y3):
                    self.set_rect(x0, y0, x1, y1, x2, y2, x3, y3)
        elif handle == 1:
            if vector_norm(x3, y3, x0, y0) and vector_norm(x3, y3, x2, y2):
                v1n = np.array((nx - x1, ny - y1))
                x0, y0 = vector_projection(v1n, x3, y3, x0, y0)
                x2, y2 = vector_projection(v1n, x3, y3, x2, y2)
                x1, y1 = nx, ny
                if _no_null_vector(x0, y0, x1, y1, x2, y2, x3, y3):
                    self.set_rect(x0, y0, x1, y1, x2, y2, x3, y3)
        elif handle == 2:
            if vector_norm(x0, y0, x1, y1) and vector_norm(x0, y0, x3, y3):
                v2n = np.array((nx - x2, ny - y2))
                x1, y1 = vector_projection(v2n, x0, y0, x1, y1)
                x3, y3 = vector_projection(v2n, x0, y0, x3, y3)
                x2, y2 = nx, ny
                if _no_null_vector(x0, y0, x1, y1, x2, y2, x3, y3):
                    self.set_rect(x0, y0, x1, y1, x2, y2, x3, y3)
        elif handle == 3:
            if vector_norm(x1, y1, x0, y0) and vector_norm(x1, y1, x2, y2):
                v3n = np.array((nx - x3, ny - y3))
                x0, y0 = vector_projection(v3n, x1, y1, x0, y0)
                x2, y2 = vector_projection(v3n, x1, y1, x2, y2)
                x3, y3 = nx, ny
                if _no_null_vector(x0, y0, x1, y1, x2, y2, x3, y3):
                    self.set_rect(x0, y0, x1, y1, x2, y2, x3, y3)
        elif handle == 4:
            x4, y4 = 0.5 * (x0 + x3), 0.5 * (y0 + y3)
            x5, y5 = 0.5 * (x1 + x2), 0.5 * (y1 + y2)
            nx, ny = x0 + nx - x4, y0 + ny - y4  # moving handle #4 to handle #0

            v10 = np.array((x0 - x1, y0 - y1))
            v12 = np.array((x2 - x1, y2 - y1))
            v10n = np.array((nx - x1, ny - y1))
            k = np.linalg.norm(v12) / np.linalg.norm(v10)
            v12n = vector_rotation(-np.pi / 2, *v10n) * k
            x2, y2 = v12n + np.array([x1, y1])
            x3, y3 = v12n + v10n + np.array([x1, y1])
            x0, y0 = nx, ny

            dx = x5 - 0.5 * (x1 + x2)
            dy = y5 - 0.5 * (y1 + y2)
            x0, y0 = x0 + dx, y0 + dy
            x1, y1 = x1 + dx, y1 + dy
            x2, y2 = x2 + dx, y2 + dy
            x3, y3 = x3 + dx, y3 + dy
            self.set_rect(x0, y0, x1, y1, x2, y2, x3, y3)
        elif handle == 5:
            x4, y4 = 0.5 * (x0 + x3), 0.5 * (y0 + y3)
            x5, y5 = 0.5 * (x1 + x2), 0.5 * (y1 + y2)
            nx, ny = x1 + nx - x5, y1 + ny - y5  # moving handle #5 to handle #1

            v01 = np.array((x1 - x0, y1 - y0))
            v03 = np.array((x3 - x0, y3 - y0))
            v01n = np.array((nx - x0, ny - y0))
            k = np.linalg.norm(v03) / np.linalg.norm(v01)
            v03n = vector_rotation(np.pi / 2, *v01n) * k
            x3, y3 = v03n + np.array([x0, y0])
            x2, y2 = v03n + v01n + np.array([x0, y0])
            x1, y1 = nx, ny

            dx = x4 - 0.5 * (x0 + x3)
            dy = y4 - 0.5 * (y0 + y3)
            x0, y0 = x0 + dx, y0 + dy
            x1, y1 = x1 + dx, y1 + dy
            x2, y2 = x2 + dx, y2 + dy
            x3, y3 = x3 + dx, y3 + dy
            self.set_rect(x0, y0, x1, y1, x2, y2, x3, y3)
        elif handle == -1:
            delta = (nx, ny) - self.points.mean(axis=0)
            self.points += delta

    def __reduce__(self) -> tuple:
        """Return the state of the object for pickling"""
        state = (self.shapeparam, self.points, self.z())
        return (self.__class__, (), state)

    def __setstate__(self, state: tuple) -> None:
        """Set the state of the object from pickling"""
        self.shapeparam, self.points, z = state
        self.setZ(z)
        self.shapeparam.update_item(self)


assert_interfaces_valid(ObliqueRectangleShape)



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

# pylint: disable=C0103

"""
Label Item builder
------------------

This module provides a set of factory functions to simplify the creation
of label items.
"""

# Note: when adding method to builder classes, please do not forget to update the
# documentation (see builder.rst file). Because of class inheritance, the methods
# are not automatically documented (otherwise, they would be sorted alphabetically,
# due to a limitation of sphinx auto-doc).

from __future__ import annotations

from collections.abc import Callable

from plotpy.config import CONF, _, make_title
from plotpy.items import (
    CurveItem,
    DataInfoLabel,
    ImageItem,
    LabelItem,
    LegendBoxItem,
    RangeComputation,
    RangeComputation2d,
    RangeInfo,
    RectangleShape,
    SelectedLegendBoxItem,
    XRangeSelection,
)
from plotpy.styles import LabelParam, LabelParamWithContents, LegendParam

# default offset positions for anchors
ANCHOR_OFFSETS = {
    "TL": (5, 5),
    "TR": (-5, 5),
    "BL": (5, -5),
    "BR": (-5, -5),
    "L": (5, 0),
    "R": (-5, 0),
    "T": (0, 5),
    "B": (0, -5),
}

LABEL_COUNT = 0


class LabelBuilder:
    """Class regrouping a set of factory functions to simplify the creation
    of label items."""

    def label(
        self,
        text: str,
        g: tuple[float, float] | str,
        c: tuple[int, int],
        anchor: str,
        title: str = "",
    ) -> LabelItem:
        """Make a label `plot item`

        Args:
            text: label text
            g: position in plot coordinates or relative position (string)
            c: position in canvas coordinates
            anchor: anchor position in relative position (string)
            title: label title. Default is ''

        Returns:
            :py:class:`.LabelItem` object

        Examples::
            make.label("Relative position", (x[0], y[0]), (10, 10), "BR")
            make.label("Absolute position", "R", (0,0), "R")
        """
        basename = _("Label")
        param = LabelParamWithContents(basename, icon="label.png")
        param.read_config(CONF, "plot", "label")
        if title:
            param.label = title
        else:
            global LABEL_COUNT
            LABEL_COUNT += 1
            param.label = make_title(basename, LABEL_COUNT)
        if isinstance(g, tuple):
            param.abspos = False
            param.xg, param.yg = g
        else:
            param.abspos = True
            param.absg = g
        if c is None:
            c = ANCHOR_OFFSETS[anchor]
        param.xc, param.yc = c
        param.anchor = anchor
        return LabelItem(text, param)

    def legend(
        self,
        anchor: str = "TR",
        c: tuple[int, int] | None = None,
        restrict_items: list | None = None,
    ) -> LegendBoxItem | SelectedLegendBoxItem:
        """Make a legend `plot item`

        Args:
            anchor: legend position in relative position (string)
            c: position in canvas coordinates
            restrict_items: list of items to be shown in legend box.
             Default is None. If None, all items are shown in legend box.
             If [], no item is shown in legend box. If [item1, item2],
             item1, item2 are shown in legend box.

        Returns:
            :py:class:`.LegendBoxItem` or
            :py:class:`.SelectedLegendBoxItem` object
        """
        param = LegendParam(_("Legend"), icon="legend.png")
        param.read_config(CONF, "plot", "legend")
        param.abspos = True
        param.absg = anchor
        param.anchor = anchor
        if c is None:
            c = ANCHOR_OFFSETS[anchor]
        param.xc, param.yc = c
        if restrict_items is None:
            return LegendBoxItem(param)
        else:
            return SelectedLegendBoxItem(param, restrict_items)

    def info_label(
        self, anchor: str, comps: list, title: str | None = None
    ) -> DataInfoLabel:
        """Make an info label `plot item`

        Args:
            anchor: anchor position. See :py:class:`.LabelParam` for details
            comps: list of :py:class:`.label.RangeComputation` objects
            title: label name. Default is None

        Returns:
            :py:class:`.DataInfoLabel` object
        """
        basename = _("Computation")
        param = LabelParam(basename, icon="label.png")
        param.read_config(CONF, "plot", "info_label")
        if title is not None:
            param.label = title
        else:
            global LABEL_COUNT
            LABEL_COUNT += 1
            param.label = make_title(basename, LABEL_COUNT)
        param.abspos = True
        param.absg = anchor
        param.anchor = anchor
        c = ANCHOR_OFFSETS[anchor]
        param.xc, param.yc = c
        return DataInfoLabel(param, comps)

    def range_info_label(
        self,
        range: XRangeSelection,
        anchor: str,
        label: str,
        function: Callable = None,
        title: str | None = None,
    ) -> DataInfoLabel:
        """Make an info label `plot item` showing an XRangeSelection object infos

        Args:
            range: range selection object
            anchor: anchor position. See :py:class:`.LabelParam` for details
            label: label name. See :py:class:`.DataInfoLabel` for details
            function: function to apply to the range selection object
             Default is None (default function is `lambda x, dx: (x, dx)`)
            title: label name. Default is None

        Returns:
            :py:class:`.DataInfoLabel` object

        Example::

            x = linspace(-10, 10, 10)
            y = sin(sin(sin(x)))
            range = make.range(-2, 2)
            disp = make.range_info_label(range, 'BL', "x = %.1f ± %.1f cm",
                                         lambda x, dx: (x, dx))
        """
        info = RangeInfo(label, range, function)
        return self.info_label(anchor, info, title=title)

    def computation(
        self,
        range: XRangeSelection,
        anchor: str,
        label: str,
        curve: CurveItem,
        function: Callable,
        title: str | None = None,
    ) -> DataInfoLabel:
        """Make a computation label `plot item`

        Args:
            range: range selection object
            anchor: anchor position. See :py:class:`.LabelParam` for details
            label: label name. See :py:class:`.DataInfoLabel` for details
            curve: curve item
            function: function to apply to the range selection object
             Default is None (default function is `lambda x, dx: (x, dx)`)

        Returns:
            :py:class:`.DataInfoLabel` object
        """
        if title is None:
            title = curve.param.label
        return self.computations(range, anchor, [(curve, label, function)], title=title)

    def computations(
        self, range: XRangeSelection, anchor: str, specs: list, title: str | None = None
    ) -> DataInfoLabel:
        """Make computation labels  `plot item`

        Args:
            range: range selection object
            anchor: anchor position. See :py:class:`.LabelParam` for details
            specs: list of (curve, label, function) tuples
            title: label name. Default is None

        Returns:
            Data info label object
        """
        comps = []
        same_curve = True
        curve0 = None
        for curve, label, function in specs:
            comp = RangeComputation(label, curve, range, function)
            comps.append(comp)
            if curve0 is None:
                curve0 = curve
            same_curve = same_curve and curve is curve0
        if title is None and same_curve:
            title = curve.param.label
        return self.info_label(anchor, comps, title=title)

    def computation2d(
        self,
        rect: RectangleShape,
        anchor: str,
        label: str,
        image: ImageItem,
        function: Callable,
        title: str | None = None,
    ) -> DataInfoLabel:
        """Make a 2D computation label `plot item`

        Args:
            rect: rectangle selection object
            anchor: anchor position. See :py:class:`.LabelParam` for details
            label: label name. See :py:class:`.DataInfoLabel` for details
            image: image item
            function: function to apply to the rectangle selection object
             Default is None (default function is `lambda x, dx: (x, dx)`)
            title: label name. Default is None

        Returns:
            Data info label object
        """
        return self.computations2d(
            rect, anchor, [(image, label, function)], title=title
        )

    def computations2d(
        self, rect: RectangleShape, anchor: str, specs: list, title: str | None = None
    ) -> DataInfoLabel:
        """Make 2D computation labels `plot item`

        Args:
            rect: rectangle selection object
            anchor: anchor position. See :py:class:`.LabelParam` for details
            specs: list of (image, label, function) tuples
            title: label name. Default is None

        Returns:
            Data info label object
        """
        comps = []
        same_image = True
        image0 = None
        for image, label, function in specs:
            comp = RangeComputation2d(label, image, rect, function)
            comps.append(comp)
            if image0 is None:
                image0 = image
            same_image = same_image and image is image0
        if title is None and same_image:
            title = image.param.label
        return self.info_label(anchor, comps, title=title)



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

"""Testing `auto_tools` plot option"""

# guitest: show

from guidata.qthelpers import qt_app_context
from numpy import linspace, sin

from plotpy.builder import make
from plotpy.tests import vistools as ptv


def test_no_auto_tools():
    """Test no auto tools"""
    x = linspace(-10, 10, 200)
    y = sin(sin(sin(x)))
    with qt_app_context(exec_loop=True):
        _win = ptv.show_items(
            [make.curve(x, y, color="b")],
            auto_tools=False,
            wintitle=test_no_auto_tools.__doc__,
        )


if __name__ == "__main__":
    test_no_auto_tools()



# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause
# (see plotpy/LICENSE for details)

"""Simple filter testing application based on PyQt and plotpy
filtertest1.py + plot manager"""

# guitest: show

import numpy as np
import scipy.ndimage as spi
import scipy.signal as sps
from guidata.configtools import get_icon
from guidata.qthelpers import qt_app_context, win32_fix_title_bar_background
from qtpy import QtWidgets as QW

import plotpy.config  # Loading icons # noqa: F401
from plotpy.builder import make
from plotpy.plot import BasePlot, BasePlotOptions
from plotpy.plot.manager import PlotManager


class FilterTestWidget(QW.QWidget):
    """
    Filter testing widget
    parent: parent widget (QWidget)
    x, y: NumPy arrays
    func: function object (the signal filter to be tested)
    """

    def __init__(self, parent, x, y, func):
        QW.QWidget.__init__(self, parent)
        self.setMinimumSize(320, 200)
        self.x = x
        self.y = y
        self.func = func
        # ---plotpy related attributes:
        self.plot = None
        self.curve_item = None
        # ---

    def setup_widget(self, title):
        # ---Create the plot widget:
        self.plot = BasePlot(self, options=BasePlotOptions(type="curve"))
        self.curve_item = make.curve([], [], color="b")
        self.plot.add_item(self.curve_item)
        self.plot.set_antialiasing(True)
        # ---

        button = QW.QPushButton("Test filter: {}".format(title))
        button.clicked.connect(self.process_data)
        vlayout = QW.QVBoxLayout()
        vlayout.addWidget(self.plot)
        vlayout.addWidget(button)
        self.setLayout(vlayout)

        self.update_curve()

    def process_data(self):
        self.y = self.func(self.y)
        self.update_curve()

    def update_curve(self):
        # ---Update curve
        self.curve_item.set_data(self.x, self.y)
        self.plot.replot()
        # ---


class WindowTest(QW.QMainWindow):
    def __init__(self):
        super().__init__()
        win32_fix_title_bar_background(self)
        self.setWindowTitle("Signal filtering 2 (plotpy)")
        self.setWindowIcon(get_icon("plotpy.svg"))

        hlayout = QW.QHBoxLayout()
        central_widget = QW.QWidget(self)
        central_widget.setLayout(hlayout)
        self.setCentralWidget(central_widget)
        # ---plotpy plot manager
        self.manager = PlotManager(self)
        # ---

    def add_plot(self, x, y, func, title):
        widget = FilterTestWidget(self, x, y, func)
        widget.setup_widget(title)
        self.centralWidget().layout().addWidget(widget)
        # ---Register plot to manager
        self.manager.add_plot(widget.plot)
        # ---

    def setup_window(self):
        # ---Add toolbar and register manager tools
        toolbar = self.addToolBar("tools")
        self.manager.add_toolbar(toolbar, id(toolbar))
        self.manager.register_all_curve_tools()
        # ---


def test_filter2():
    """Testing this simple Qt/plotpy example"""
    x = np.linspace(-10, 10, 500)
    y = np.random.rand(len(x)) + 5 * np.sin(2 * x**2) / x
    with qt_app_context(exec_loop=True):
        win = WindowTest()
        win.add_plot(x, y, lambda x: spi.gaussian_filter1d(x, 1.0), "Gaussian")
        win.add_plot(x, y, sps.wiener, "Wiener")
        win.setup_window()
        win.show()


if __name__ == "__main__":
    test_filter2()



# -*- coding: utf-8 -*-

from __future__ import annotations

import math
from typing import TYPE_CHECKING

from guidata.configtools import get_icon
from guidata.dataset import update_dataset
from guidata.utils.misc import assert_interfaces_valid
from qtpy import QtCore as QC
from qwt import QwtPlotMarker

from plotpy.config import CONF, _
from plotpy.coords import canvas_to_axes
from plotpy.interfaces import IBasePlotItem, IShapeItemType
from plotpy.styles.base import MARKERSTYLES
from plotpy.styles.shape import MarkerParam

if TYPE_CHECKING:
    from collections.abc import Callable

    import guidata.io
    import qwt.scale_map
    from qtpy.QtCore import QPointF, QRectF
    from qtpy.QtGui import QPainter

    from plotpy.interfaces import IItemType
    from plotpy.styles.base import ItemParameters


class Marker(QwtPlotMarker):
    """Marker shape

    Args:
        label_cb: Label callback (must return a string, takes x and y as arguments)
        constraint_cb: Constraint callback (must return a tuple (x, y),
         takes x and y as arguments)
        markerparam: Marker parameters

    .. note::

        Marker class derives from QwtPlotMarker, which is a QwtPlotItem. That is
        why AbstractShape methods are re-implemented here.
    """

    __implements__ = (IBasePlotItem,)
    _readonly = False
    _private = False
    _can_select = True
    _can_resize = True
    _can_rotate = False
    _can_move = True

    def __init__(
        self,
        label_cb: Callable | None = None,
        constraint_cb: Callable | None = None,
        markerparam: MarkerParam = None,
    ) -> None:
        super().__init__()
        self._pending_center_handle = None
        self.selected = False
        self.label_cb = label_cb
        if constraint_cb is None:
            constraint_cb = self.center_handle
        self.constraint_cb = constraint_cb
        if markerparam is None:
            self.markerparam = MarkerParam(_("Marker"))
            self.markerparam.read_config(CONF, "plot", "marker/cursor")
        else:
            self.markerparam = markerparam
        self.markerparam.update_item(self)
        self.setIcon(get_icon("marker.png"))

    def __reduce__(self) -> tuple[type, tuple, tuple]:
        """Return state information for pickling"""
        self.markerparam.update_param(self)
        state = (self.markerparam, self.xValue(), self.yValue(), self.z())
        return (Marker, (), state)

    def __setstate__(self, state: tuple) -> None:
        """Restore state information from pickled state"""
        self.markerparam, xvalue, yvalue, z = state
        self.setXValue(xvalue)
        self.setYValue(yvalue)
        self.setZ(z)
        self.markerparam.update_item(self)

    def serialize(
        self,
        writer: guidata.io.HDF5Writer | guidata.io.INIWriter | guidata.io.JSONWriter,
    ) -> None:
        """Serialize object to HDF5 writer

        Args:
            writer: HDF5, INI or JSON writer
        """
        self.markerparam.update_param(self)
        writer.write(self.markerparam, group_name="markerparam")
        writer.write(self.xValue(), group_name="x")
        writer.write(self.yValue(), group_name="y")
        writer.write(self.z(), group_name="z")

    def deserialize(
        self,
        reader: guidata.io.HDF5Reader | guidata.io.INIReader | guidata.io.JSONReader,
    ) -> None:
        """Deserialize object from HDF5 reader

        Args:
            reader: HDF5, INI or JSON reader
        """
        self.markerparam = MarkerParam(_("Marker"), icon="marker.png")
        reader.read("markerparam", instance=self.markerparam)
        self.markerparam.update_item(self)
        self.setXValue(reader.read("x"))
        self.setYValue(reader.read("y"))
        self.setZ(reader.read("z"))

    # ------QwtPlotItem API------------------------------------------------------
    def draw(
        self,
        painter: QPainter,
        xMap: qwt.scale_map.QwtScaleMap,
        yMap: qwt.scale_map.QwtScaleMap,
        canvasRect: QRectF,
    ) -> None:
        """Draw the item

        Args:
            painter: Painter
            xMap: X axis scale map
            yMap: Y axis scale map
            canvasRect: Canvas rectangle
        """
        if self._pending_center_handle:
            x, y = self.center_handle(self.xValue(), self.yValue())
            self.setValue(x, y)
        self.update_label()
        QwtPlotMarker.draw(self, painter, xMap, yMap, canvasRect)

    # ------IBasePlotItem API----------------------------------------------------
    def set_selectable(self, state: bool) -> None:
        """Set item selectable state

        Args:
            state: True if item is selectable, False otherwise
        """
        self._can_select = state

    def set_resizable(self, state: bool) -> None:
        """Set item resizable state
        (or any action triggered when moving an handle, e.g. rotation)

        Args:
            state: True if item is resizable, False otherwise
        """
        self._can_resize = state

    def set_movable(self, state: bool) -> None:
        """Set item movable state

        Args:
            state: True if item is movable, False otherwise
        """
        self._can_move = state

    def set_rotatable(self, state: bool) -> None:
        """Set item rotatable state

        Args:
            state: True if item is rotatable, False otherwise
        """
        self._can_rotate = state

    def can_select(self) -> bool:
        """
        Returns True if this item can be selected

        Returns:
            bool: True if item can be selected, False otherwise
        """
        return self._can_select

    def can_resize(self) -> bool:
        """
        Returns True if this item can be resized

        Returns:
            bool: True if item can be resized, False otherwise
        """
        return self._can_resize

    def can_rotate(self) -> bool:
        """
        Returns True if this item can be rotated

        Returns:
            bool: True if item can be rotated, False otherwise
        """
        return self._can_rotate

    def can_move(self) -> bool:
        """
        Returns True if this item can be moved

        Returns:
            bool: True if item can be moved, False otherwise
        """
        return self._can_move

    def types(self) -> tuple[type[IItemType], ...]:
        """Returns a group or category for this item.
        This should be a tuple of class objects inheriting from IItemType

        Returns:
            tuple: Tuple of class objects inheriting from IItemType
        """
        return (IShapeItemType,)

    def set_readonly(self, state: bool) -> None:
        """Set object readonly state

        Args:
            state: True if object is readonly, False otherwise
        """
        self._readonly = state

    def is_readonly(self) -> bool:
        """Return object readonly state

        Returns:
            bool: True if object is readonly, False otherwise
        """
        return self._readonly

    def set_private(self, state: bool) -> None:
        """Set object as private

        Args:
            state: True if object is private, False otherwise
        """
        self._private = state

    def is_private(self) -> bool:
        """Return True if object is private

        Returns:
            bool: True if object is private, False otherwise
        """
        return self._private

    def select(self) -> None:
        """
        Select the object and eventually change its appearance to highlight the
        fact that it's selected
        """
        if self.selected:
            # Already selected
            return
        self.selected = True
        self.markerparam.update_item(self)
        self.invalidate_plot()

    def unselect(self) -> None:
        """
        Unselect the object and eventually restore its original appearance to
        highlight the fact that it's not selected anymore
        """
        self.selected = False
        self.markerparam.update_item(self)
        self.invalidate_plot()

    def hit_test(self, pos: QPointF) -> tuple[float, float, bool, None]:
        """Return a tuple (distance, attach point, inside, other_object)

        Args:
            pos: Position

        Returns:
            tuple: Tuple with four elements: (distance, attach point, inside,
             other_object).

        Description of the returned values:

        * distance: distance in pixels (canvas coordinates) to the closest
           attach point
        * attach point: handle of the attach point
        * inside: True if the mouse button has been clicked inside the object
        * other_object: if not None, reference of the object which will be
           considered as hit instead of self
        """
        plot = self.plot()
        xc, yc = pos.x(), pos.y()
        x = plot.transform(self.xAxis(), self.xValue())
        y = plot.transform(self.yAxis(), self.yValue())
        ms = self.markerparam.markerstyle
        # The following assert has no purpose except reminding that the
        # markerstyle is one of the MARKERSTYLES dictionary values, in case
        # this dictionary evolves in the future (this should not fail):
        assert ms in list(MARKERSTYLES.values())
        if ms == "NoLine":
            return math.sqrt((x - xc) ** 2 + (y - yc) ** 2), 0, False, None
        elif ms == "HLine":
            return math.sqrt((y - yc) ** 2), 0, False, None
        elif ms == "VLine":
            return math.sqrt((x - xc) ** 2), 0, False, None
        elif ms == "Cross":
            return math.sqrt(min((x - xc) ** 2, (y - yc) ** 2)), 0, False, None

    def update_item_parameters(self) -> None:
        """Update item parameters (dataset) from object properties"""
        self.markerparam.update_param(self)

    def get_item_parameters(self, itemparams: ItemParameters) -> None:
        """
        Appends datasets to the list of DataSets describing the parameters
        used to customize apearance of this item

        Args:
            itemparams: Item parameters
        """
        self.update_item_parameters()
        itemparams.add("MarkerParam", self, self.markerparam)

    def set_item_parameters(self, itemparams):
        """
        Change the appearance of this item according
        to the parameter set provided

        params is a list of Datasets of the same types as those returned
        by get_item_parameters
        """
        update_dataset(
            self.markerparam, itemparams.get("MarkerParam"), visible_only=True
        )
        self.markerparam.update_item(self)
        if self.selected:
            self.select()

    def move_local_point_to(self, handle: int, pos: QPointF, ctrl: bool = None) -> None:
        """Move a handle as returned by hit_test to the new position

        Args:
            handle: Handle
            pos: Position
            ctrl: True if <Ctrl> button is being pressed, False otherwise
        """
        x, y = canvas_to_axes(self, pos)
        self.set_pos(x, y)

    def move_local_shape(self, old_pos: QPointF, new_pos: QPointF) -> None:
        """Translate the shape such that old_pos becomes new_pos in canvas coordinates

        Args:
            old_pos: Old position
            new_pos: New position
        """
        # This methods is never called because marker is not a shape (but a point)

    def move_with_selection(self, delta_x: float, delta_y: float) -> None:
        """Translate the item together with other selected items

        Args:
            delta_x: Translation in plot coordinates along x-axis
            delta_y: Translation in plot coordinates along y-axis
        """
        # This methods is never called because marker is not a shape (but a point)

    # ------Public API-----------------------------------------------------------
    def set_style(self, section: str, option: str) -> None:
        """Set style for this item

        Args:
            section: Section
            option: Option
        """
        self.markerparam.read_config(CONF, section, option)
        self.markerparam.update_item(self)

    def set_pos(self, x: float | None = None, y: float | None = None) -> None:
        """Set marker position

        Args:
            x: X value (if None, use current value)
            y: Y value (if None, use current value)
        """
        if x is None:
            x = self.xValue()
        if y is None:
            y = self.yValue()
        if self.constraint_cb:
            x, y = self.constraint_cb(x, y)
        self.setValue(x, y)
        if self.plot():
            self.plot().SIG_MARKER_CHANGED.emit(self)

    def get_pos(self) -> tuple[float, float]:
        """Get marker position

        Returns:
            Tuple with two elements (x, y)
        """
        return self.xValue(), self.yValue()

    def set_markerstyle(self, style: str | int | None) -> None:
        """Set marker style

        Args:
            style: Marker style
        """
        param = self.markerparam
        param.set_markerstyle(style)
        param.update_item(self)

    def is_vertical(self) -> bool:
        """Is it a vertical cursor?

        Returns:
            True if this is a vertical cursor
        """
        return self.lineStyle() == QwtPlotMarker.VLine

    def is_horizontal(self) -> bool:
        """Is it a horizontal cursor?

        Returns:
            True if this is a horizontal cursor
        """
        return self.lineStyle() == QwtPlotMarker.HLine

    def center_handle(self, x: float, y: float) -> tuple[float, float]:
        r"""Center cursor handle depending on marker style (\|, -)

        Args:
            x: X value
            y: Y value

        Returns:
            Tuple with two elements (x, y)
        """
        plot = self.plot()
        if plot is None:
            self._pending_center_handle = True
        else:
            self._pending_center_handle = False
            if self.is_vertical():
                ymap = plot.canvasMap(self.yAxis())
                y_top, y_bottom = ymap.s1(), ymap.s2()
                y = 0.5 * (y_top + y_bottom)
            elif self.is_horizontal():
                xmap = plot.canvasMap(self.xAxis())
                x_left, x_right = xmap.s1(), xmap.s2()
                x = 0.5 * (x_left + x_right)
        return x, y

    def invalidate_plot(self) -> None:
        """Invalidate the plot to force a redraw"""
        plot = self.plot()
        if plot is not None:
            plot.invalidate()

    def update_label(self) -> None:
        """Update label"""
        x, y = self.xValue(), self.yValue()
        if self.label_cb:
            label = self.label_cb(x, y)
            if label is None:
                return
        elif self.is_vertical():
            label = f"x = {x:g}"
        elif self.is_horizontal():
            label = f"y = {y:g}"
        else:
            label = f"x = {x:g}<br>y = {y:g}"
        text = self.label()
        text.setText(label)
        self.setLabel(text)
        plot = self.plot()
        if plot is not None:
            xaxis = plot.axisScaleDiv(self.xAxis())
            if x < (xaxis.upperBound() + xaxis.lowerBound()) / 2:
                hor_alignment = QC.Qt.AlignRight
            else:
                hor_alignment = QC.Qt.AlignLeft
            yaxis = plot.axisScaleDiv(self.yAxis())
            ymap = plot.canvasMap(self.yAxis())
            y_top, y_bottom = ymap.s1(), ymap.s2()
            if y < 0.5 * (yaxis.upperBound() + yaxis.lowerBound()):
                if y_top > y_bottom:
                    ver_alignment = QC.Qt.AlignBottom
                else:
                    ver_alignment = QC.Qt.AlignTop
            else:
                if y_top > y_bottom:
                    ver_alignment = QC.Qt.AlignTop
                else:
                    ver_alignment = QC.Qt.AlignBottom
            self.setLabelAlignment(hor_alignment | ver_alignment)


assert_interfaces_valid(Marker)
